/*************************************************************************** 
 *                                                                         *
 *    File:         TeD.h                                                  *
 *                                                                         *
 *    Descrizione:  Prototipi e strutture dati usati nel sistema TeD       *
 *                  X-Windows.                                             *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati                              *
 *                                                                         *
 *    Date:         8 Ott 1994                                             *
 *                                                                         *
 ***************************************************************************/

#include <qpainter.h>                // Painting
#include <qpoint.h>                  // Points
#include <qrect.h>                   // Rectangles
#include <qpaintd.h>                 // Paint device
#include <qwidget.h>                 // Widgets

/*************************************************************************** 
 *                                                                         *
 *  Parametri                                                              *
 *                                                                         *
 ***************************************************************************/

#define IDM_FILE             2001                // Menu Item
#define IDM_NEW              2002                // Menu Item
#define IDM_OPEN             2003                // Menu Item
#define IDM_SAVE             2004                // Menu Item
#define IDM_SAVEAS           2005                // Menu Item
#define IDM_AUTOSAVE         2006                // Menu Item
#define IDM_QUIT             2008                // Menu Item
#define IDM_EDIT             2009                // Menu Item
#define IDM_CUT              2010                // Menu Item
#define IDM_COPY             2011                // Menu Item
#define IDM_PASTE            2012                // Menu Item
#define IDM_CLEAR            2013                // Menu Item
#define IDM_REFORMAT         2014                // Menu Item
#define IDM_OPTIONS          2015                // Menu Item
#define IDM_FIND             2016                // Menu Item
#define IDM_REPLACE          2017                // Menu Item
#define IDM_GOTO             2018                // Menu Item
#define IDM_HELP             2019                // Menu Item
#define IDM_PRODUCTINFO      2024                // Menu Item
#define IDM_INSERT           2025                // Menu Item
#define IDM_FONT             2026                // Menu Item
#define IDM_FAMILY           2027                // Menu Item
#define IDM_SIZE             2028                // Menu Item
#define IDM_SPELL            2029                // Menu Item

/***************************************************************************
 *                                                                         *
 *  Costanti globali                                                       *
 *                                                                         *
 ***************************************************************************/

#define SOFTRETURN "\267"   /* simbolo soft return */
#define CONTINUATION "\273" /* simbolo continuation */
#define HARDRETURN "\266"   /* simbolo hard return */
#define ENDLINE "\12"       /* carattere fine riga su file */
#define EOL 10              /* codice ASCII carattere fine riga sul file */
#define CCHMAXPATH 255      /* Lunghezza massima path */
#define FIRSTCHAR 0L        /* codice ASCII primo carattere possibile */
#define CURWIDTH 2L         /* larghezza cursore */
#define TID_MOUSESCROLL 1   /* ID del timer del mouse */
#define THREAD_STACK 8192L  /* lunghezza dello stack per thread di ricerca */
#define TRUE 1              /* true */
#define FALSE 0             /* false */
#define MBID_YES 1          /* risposta yes message box */
#define MBID_NO 2           /* risposta no message box */
#define MBID_CANCEL 3       /* risposta cancel message box */
#define MB_OK 1             /* OK message box */
#define MB_YESNOCANCEL 2    /* Yes No Cancel message box */
#define SCROLLW 16          // Width scrollbars
#define BOTTOMW 20          // Width bottom line
#define SPACE 4             // Width space

/***************************************************************************
 *                                                                         *
 *  Strutture dati                                                         *
 *                                                                         *
 ***************************************************************************/

enum ret {soft,hard,cont};   /* ret specifica il tipo di ritorno a capo */

struct row
   {
      char text[ROWLEN+1];   /* ogni riga ha ROWLEN caratteri */
      enum ret carriage;     /* tipo di ritorno a capo */
      struct row *next;      /* puntatore alla riga seguente */
      struct row *prev;      /* puntatore alla riga precedente */
   };

struct cur
   {
    int x;                  /* coordinata x cursore */
    int y;                  /* coordinata y cursore */
    };


/*************************************************************************** 
 *                                                                         *
 *  Modello funzioni                                                       *
 *                                                                         *
 ***************************************************************************/

struct row *add(struct row *line);
struct row *del_line(struct row *line);
void bk_key(MainWindow *hwnd);
void CentrateCursor(MainWindow *hwnd);
void cen_key(void);
char check_autosave(void);
char check_clear(void);
char check_copy(void);
char check_cut(void);
char check_find(void);
char check_goto(void);
char check_paste(void);
char check_reformat(void);
char check_replace(void);
char check_save(void);
char check_spell(void);
void chm_key(void);
void cl_key(void);
void ComputeLineWidth(char *string,int *width);
char CopyMark(MainWindow *hwnd);
void cr_key(void);
void CursorAt(int x, int y);
void del_key(MainWindow *hwnd);
void DelMark(MainWindow *hwnd, char *text);
char Deselect(MainWindow *hwnd);
void DisplayLine(QPainter *p, QPoint *pt, const QRect *ur, struct row *pointer,
                 char *disp, char high, unsigned int high_start,
                 unsigned int high_stop);
void dn_key(void);
void DownUpdate(unsigned int nline, MainWindow *hwnd);
void end_key(void);
void ent_key(MainWindow *hwnd);
struct row *erase(struct row *line);
enum ret extract(struct row *punt,char *string);
char FileOpen(char* pszFullFile);
struct row *fill(struct row *source,char *temp,int *n,char *string,
                 unsigned int len,unsigned int *pos,unsigned int *phase);
void GetCharAt(int x, int y, unsigned int *prow, unsigned int *pline,
               unsigned int *pcol, struct row **pointer);
void go_to(unsigned int jmp);
void hm_key(void);
void IniName(char *environ[], char *out);
void init(struct row *line);
void insert(char usChar, char fInsert, MainWindow *hwnd);
void InvalidateArea(MainWindow *hwnd, unsigned int row_start,
                    unsigned int column_start,
                    struct row *point_start, unsigned int row_stop,
                    unsigned int column_stop, struct row *point_stop);
int justfy(MainWindow *hwnd, unsigned int first_modified,
           unsigned int last_modified, char *ins_line);
struct row *LineManager(int i,struct row *line);
void LineDown(MainWindow *hwnd);
void LineLeft(MainWindow *hwnd);
void LineRight(MainWindow *hwnd);
void LinesUpdate(unsigned int nstart, unsigned int nstop, MainWindow *hwnd);
void LineUp(MainWindow *hwnd);
void l_key(void);
void MenuSave(MainWindow *hwnd);
void modify(MainWindow *hwnd);
void mov_cur(int rel);
char NextCharPar(unsigned int *row, unsigned int *line, unsigned int *column,
                 struct row **now);
void next_line(int *first_line);
void ParseName(char *in,char *out);
char PasteClipboard(MainWindow *hwnd);
void pdn_key(void);
char PrecCharPar(unsigned int *row, unsigned int *line, unsigned int *column,
                 struct row **now);
void PrintText(QPainter *p, QPoint *pt, const QRect *r, char *text, int num);
void pup_key(void);
void ReadIniFile(char *fname, unsigned int *as, char *fontname, int *size,
                 QSize *Size);
void r_key(void);
enum ret ReadFormattedString(char *in, char *out, unsigned int *pos,
                             unsigned int len);
void Reformat(MainWindow *hwnd);
char SaveFile(char *name);
void Scroll (QPaintDevice *dev, int dd, int dir);
char search_string(find_text *ft);
void Select(MainWindow *hwnd);
char SetClipboardData(MainWindow *hwnd, char *pMem);
void SetCursor(MainWindow *hwnd);
char *StringFromClipboard(MainWindow *hwnd);
void TextSelect(MainWindow *hwnd, char fsel, int x, int y, char fOutside);
void up_key(void);
void ValidateCursor(MainWindow *hwnd);
void VideoRegionUpdate(unsigned int nline, char *string,
                       MainWindow *hwnd, unsigned int first_modified,
                       unsigned int last_modified);
void VideoUpdate(unsigned int nline, char *string, MainWindow *hwnd,
                 unsigned int first_modified);
void WordSelect(MainWindow *hwnd, int x, int y);
char WriteIniFile(char *fname, unsigned int as, char *fontname, int size,
                  QSize Size);
int writeline(struct row *dest, unsigned int ndest, char *temp);
void WriteString(char *string, enum ret carriage);

