/***************************************************************************
 *                                                                         *
 *    File:         TeDAdd.cpp                                             *
 *                                                                         *
 *    Descrizione:  Aggiunta ed eliminazione di una riga del sistema       *
 *                  TeD X-Windows                                          *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Data:         29 Ago 1994                                            *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "TeDFrame.h"
#include "TeD.h"

/*
** Funzione di allocazione di una nuova riga. Tutti i puntatori vengono aggiornati.
** I contatori del numero di linee vengono incrementati se l'allocazione va a buon
** fine.
*/

struct row *add(struct row *line)       /* line e' il puntatore alla struttura riga */
{                                       /* dopo la quale va aggiunta la nuova riga */

  struct row *newline;        /* puntatore alla nuova riga */
  extern struct row *last;    /* puntatore globale all'ultima riga */
  extern unsigned int t_len;  /* contatore globale numero righe testo */
  extern unsigned int t_line; /* contatore globale numero linee testo */

  /* Alloca la nuova riga */
  
  newline = (struct row *) LineManager(1,(struct row *) NULL);
  if (newline == NULL) return NULL;     /* manca memoria */

  init(newline);   /* Inizializza la nuova riga */
  ++t_len;         /* Incrementa il numero di righe del testo */
  ++t_line;        /* Incrementa il numero di linee del testo */
  
  /* Aggiorna i puntatori delle righe */
  
  newline->prev = line;           /* riga precedente a quella nuova */
  newline->next = line->next;     /* riga seguente a quella nuova   */
  line->next = newline;           /* riga seguente a quella vecchia */
  
  if (newline->next)                  /* La riga aggiunta non e' l'ultima */
    newline->next->prev = newline;    /* riga precedente a quella vecchia */
  else
    last = newline;               /* puntatore all'ultima riga del testo */

  return newline;                 /* torna puntatore alla riga aggiunta */
}


/*
** Funzione di distruzione di una riga. I puntatori non vengono aggiornati. Ritorna
** il puntatore alla riga precedente quella cancellata.
*/

struct row *del_line(struct row *line)
{                                        /* line e' il puntatore della struttura riga */
                                         /* che deve essere distrutta */
  return(LineManager(0,line));
}

/*
** Funzione di allocazione o di deallocazione di una riga. Il parametro i indica se
** la riga va distrutta (i=0) o creata (i#0). Nel caso di creazione di riga ritorna
** l'indirizzo della riga creata. Nel caso di distruzione, il puntatore line indica
** l'indirizzo della riga da distruggere e la funzione ritorna l'indirizzo della
** riga precedente a quella distrutta.
**
** N.B. La riga effettivamente distrutta e' l'ultima allocata: eventuali puntatori
** all'infuori di <actual>, <video> e <last>, che vengono aggiornati, possono
** essere invalidati dopo la distruzione della riga.
*/

struct row *LineManager(int i,struct row *line)
{
   static char** pulLast = (char **) NULL;  /* puntatore all'ultima pagina allocata */
   static char** pulGuard = (char **) NULL; /* puntatore alla pagina di guardia */
   int LastElmOffset;                       /* offset ultimo elemento di una pagina */
   static int Offset = 0;                   /* offset iniziale */
   struct row *swap;                        /* puntatore per scambiare righe */
   struct row *temp;                        /* puntatore temporaneo */
   extern struct row *last;                 /* puntatore ultima riga */
   extern struct row *actual;               /* puntatore riga attuale */
   extern struct row *video;                /* puntatore prima riga video */
   int ulBytes = 4096;                      /* lunghezza pagina */

   LastElmOffset = (ulBytes-sizeof(char))/sizeof(struct row);
   if (i == 0) {
      /* cancellazione riga */
      temp = line->prev;            /* indirizzo riga precedente */
      if ((swap = ((struct row *)(pulLast+1))+Offset) != line) {
         /* La riga da cancellare non e' l'ultima */

         /* copia ultima riga allocata nella riga puntata da line */
         strcpy(line->text,swap->text);
         line->carriage = swap->carriage;
         line->next = swap->next;
         line->prev = swap->prev;

         /* aggiusta i puntatori delle righe adiacenti a swap */
         line->prev->next = line;      /* aggiusta puntatore della riga precedente */
         if (line->next)               /* aggiusta puntatore della riga successiva */
            line->next->prev = line;
         if (actual == swap) actual = line;  /* aggiornamento puntatore attuale */
         if (video == swap) video = line;    /* aggiornamento puntatore prima riga video */
         if (last == swap) last = line;      /* aggiornamento puntatore ultima riga */
         if (temp == swap) temp = line;      /* aggiornamento puntatore riga precedente */
      }
      /* Cancellazione ultima riga allocata */
      --Offset;                             /* decrementa offset */
      if (Offset < 0) {
         /* Eliminazione ultimo elemento pagina */
         if (pulGuard) {
            /* Eliminazione pagina di guardia */
            free(pulGuard);
         } /* endif */
         if (*pulLast) {
            /* Creazione pagina di guardia e retrocessione alla pagina precedente */
            pulGuard = pulLast;
            pulLast = (char **) *pulGuard;
            Offset = LastElmOffset-1;
         } else {
            /* Prima pagina allocata: non viene deallocata */
            pulGuard = (char **) NULL;
         } /* endif */
      } /* endif */
   return(temp);
   }
   /* creazione riga */

   if (pulLast == (char **) NULL || Offset > LastElmOffset-2) {
      /* Allocazione di una nuova pagina */
      if (pulGuard) {
         /* Esiste la pagina di guardia */
         pulLast = pulGuard;                 /* ultima pagina allocata */
         pulGuard = (char **) NULL;          /* manca pagina di guardia */
         Offset = 0;                         /* si suppone che in una pagina stia */
         return((struct row *)(pulLast+1));  /* almeno una riga! */
      } else {                                                                     
         /* Non esiste la pagina di guardia */
         pulGuard = (char **) malloc(ulBytes);    /* alloca nuova pagina */
         if (!pulGuard) {
            /* Errore di allocazione */
            return ((struct row *) NULL);
         }
         /* Allocazione riuscita */
         *pulGuard = (char *)pulLast;          /* memorizza inizio pagina precedente */
         pulLast = pulGuard;                   /* ultima pagina allocata */
         pulGuard = (char **) NULL;            /* manca pagina di guardia */
         Offset = 0;                           /* si suppone che in una pagina stia */
         return((struct row *)(pulLast+1));    /* almeno una riga! */
      } /* endif */
   } else {
      /* La pagina esiste gia' */
      ++Offset;                                    /* incrementa offset */
      return(((struct row *)(pulLast+1))+Offset);  /* ritorna indirizzo nuova riga */
   } /* endif */
}