/***************************************************************************
 *                                                                         *
 *    File:         TeDAutosave.cpp                                        *
 *                                                                         *
 *    Description:  Autosave manager of X-Window TeD                       *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1996.                     *
 *                  All rights reserved.                                   *
 *                                                                         *
 *    Date:         14 Jan 1996                                            *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>                       // Standard I/O routines
#include <qlabel.h>                      // Labels
#include <qdialog.h>                     // Dialogs
#include <qframe.h>                      // Frames
#include <qpushbt.h>                     // Push buttons
#include <qlined.h>                      // Line edit
#include <string.h>                      // Strings
#include "TeDFrame.h"
#include "TeD.h"


// Autosave window constructor

WinAutosave::WinAutosave(QWidget* parent, const char* name, WFlags f)
        : QDialog (0, name, TRUE, f)
{

  extern unsigned int autosave;              // Autosave threshold
  char temp[33];                             // Temporary string

  // Save parent window
  hwnd = parent;

  // Sets frame & separator
  frame = new QFrame(this);
  separator = new QFrame(this);
  frame->setFrameStyle( QFrame::Panel | QFrame::Raised );
  separator->setFrameStyle( QFrame::HLine | QFrame::Sunken );

  // Set labels
  Mlabel1 = new QLabel(this);
  Mlabel1->setText("Save every: ");
  Mlabel1->setAlignment(AlignCenter);
  Mlabel1->setAutoResize(TRUE);
  Mlabel2 = new QLabel(this);
  Mlabel2->setText("changes");
  Mlabel2->setAlignment(AlignCenter);
  Mlabel2->setAutoResize(TRUE);
  
  // Sets line edit
  line = new QLineEdit(this);
  line->setMaxLength(32);                        // Max 32 characters
  sprintf(temp, "%d", autosave);
  line->setText(temp);

  // Sets buttons
  Ok = new QPushButton(this);
  Cancel = new QPushButton(this);
  Ok->setText("Ok");
  Cancel->setText("Cancel");

  // Connect button signals
  connect(Ok, SIGNAL(clicked()), SLOT(ok_clicked()));
  connect(Cancel, SIGNAL(clicked()), SLOT(cancel_clicked()));

  // Focus to line editor
  line->setFocus();

  // Resize window & show
  resize(240, 104);
  show();
}


// Autosave window destructor

WinAutosave::~WinAutosave()
{
  // Free storage

  delete Mlabel1;                // Message box label
  delete Mlabel2;                // Message box label
  delete line;                   // Line edit
  delete Ok;                     // Ok button
  delete Cancel;                 // Cancel button
  delete separator;              // Separator line
  delete frame;                  // Frame
}


// Autosave window resize event

void WinAutosave::resizeEvent( QResizeEvent* )
{
  const int space = 20;
  const int sspace = 10;
  const int bspace = 30;
  const int hei = 25;
  int blen = (this->width() - 3 * bspace)/2;

  // Sets frame geometry
  frame->setGeometry(0, 0, this->width(), this->height());

  // Sets separator geometry
  separator->setGeometry(0, bspace + hei , this->width(), 4);  

  // Set labels geometry
  Mlabel1->setGeometry(sspace, sspace, 80, hei);
  Mlabel2->setGeometry(width() - sspace - 50, sspace, 50, hei);

  // Sets line edit geometry
  line->setGeometry(space + 70, sspace, width() - bspace - 130, 
                    hei);

  // Sets buttons geometry
  Ok->setGeometry (bspace, this->height() - sspace - hei, blen, hei);
  Cancel->setGeometry (this->width() - bspace - blen,
                       this->height() - sspace - hei, blen, hei);

}


// Ok button pressed

void WinAutosave::ok_clicked()
{
  extern unsigned int autosave;
  extern unsigned int n_mod;

  // Gets autosave value & save if necessary
  if (sscanf(line->text(), "%d", &autosave)) {
    if (autosave && n_mod >= autosave)
      MenuSave((MainWindow *)hwnd);
    close (FALSE);                    // Close window
    }
}


// Cancel button pressed

void WinAutosave::cancel_clicked()
{

  // Close window
  close (FALSE);
}
