/***************************************************************************
 *                                                                         *
 *    File:         TeDCopy.cpp                                            *
 *                                                                         *
 *    Description:  Clipboard copy of X-Window TeD                         *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Date:         29 Aug 1994                                            *
 *                                                                         *
 ***************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "TeDFrame.h"
#include "TeD.h"


/*
** Funzione per elaborare la stringa <string> secondo il tipo di
** ritorno carrello <carriage>. Modifica la stringa in ingresso.
** N.B. La stringa cui punta <string> deve poter contenere almeno 2
** caratteri in piu' della stringa originale.
*/

void WriteString(char *string, enum ret carriage)
{
  if (carriage == cont)                    // Continuation mark 
    return;
  else if (carriage == soft) {             // Soft return 
    strcat(string," ");                    // Aggiunge spazio in coda 
    strcat(string, ENDLINE);               // Aggiunge carattere fine riga 
    return;
    }
  else {                                   // Hard return 
    register int i;                        // Contatore ausiliario 
    for (i = strlen(string) - 1; i >= 0; --i)
      if (*(string + i) != 32) break;
    *(string + i + 1) = (char) NULL;        // Elimina spazi in coda 
    strcat(string, ENDLINE);                // Aggiunge carattere fine riga 
    return;
     }
}



/*
** Funzione per copiare l'area marcata nella clipboard. Ritorna TRUE in caso
** di esito positivo, FALSE altrimenti. <hwnd> e' l'handle della finestra
** client.
*/

char CopyMark(MainWindow *hwnd)
{
  extern char fmark;                       // Presenza selezione 
  extern unsigned int cursor_row;          // Riga cursore 
  extern unsigned int cursor_column;       // Colonna cursore 
  extern struct row *actual;               // Puntatore cursore 
  extern unsigned int mark_row_end;        // Riga fine marcatura 
  extern unsigned int mark_column_end;     // Colonna fine marcatura 
  extern struct row *mark_pointer_end;     // Puntatore fine marcatura 
  unsigned int begin_row;                  // Riga inizio copia 
  unsigned int begin_column;               // Colonna inizio copia 
  struct row *begin;                       // Puntatore inizio copia 
  unsigned int end_row;                    // Riga fine copia 
  unsigned int end_column;                 // Colonna fine copia 
  int ulMem;                               // Lunghezza stringa da allocare 
  char *pMem;                              // Puntatore memoria 
  char out[ROWLEN + 2];                    // Riga in composizione 
  register unsigned int i,j;               // Contatori ausiliari 
  enum ret carriage;                       // Ritorno carrello ultima riga 


  if(!fmark)                                 // Testo non selezionato 
     return(FALSE);

  // Ordina la regione da copiare 
  if (cursor_row < mark_row_end) {
     begin_row = cursor_row;
     end_row = mark_row_end;
     begin_column = cursor_column;
     end_column = mark_column_end;
     begin = actual;
     }
  else if (cursor_row > mark_row_end) {
     begin_row = mark_row_end;
     end_row = cursor_row;
     begin_column = mark_column_end;
     end_column = cursor_column;
     begin = mark_pointer_end;
     }
  else if (cursor_column < mark_column_end) {
    begin_row = end_row = cursor_row;
    begin_column = cursor_column;
    end_column = mark_column_end;
    begin = actual;
    }
  else {
    begin_row = end_row = cursor_row;
    begin_column = mark_column_end;
    end_column = cursor_column;
    begin = actual;
    }

  // Alloca blocco di memoria condivisibile 
  ulMem = ((end_row - begin_row + 1) * (ROWLEN + 2) + 1) * sizeof(char);
  if ( !(pMem = (char *) malloc(ulMem)) )
    return(FALSE);

  // Copia l'area selezionata 
  i = 0;
  // Copia dalla prima riga alla penultima 
  for (j = begin_row; j < end_row; ++j) {
    if (begin_column <= strlen(begin->text)) {
      strcpy(out,begin->text + begin_column);
      WriteString(out,begin->carriage);
      strcpy(pMem + i,out);
      i += strlen(out);
      }
    begin = begin->next;
    begin_column = 0;
    }
  // Copia ultima riga 
  strcpy(out,begin->text + begin_column);
  if (end_column <= strlen(begin->text)) {
    carriage = cont;
    *(out + end_column - begin_column) = (char)NULL;
    }
  else
    carriage = begin->carriage;
  WriteString(out,carriage);
  strcpy(pMem + i,out);

  // Send data to clipboard
  if (!SetClipboardData(hwnd, pMem))
    return(FALSE);

  // Dealloca memoria condivisibile 
  free(pMem);

  return(TRUE);
}
