/******************************************************************
************************* Cursor class ****************************
******************************************************************/

#include <qwidget.h>                       // Widget class
#include <qobject.h>                       // Object class
#include <qpainter.h>                      // Painter class
#include <qevent.h>                        // Event handler
#include <qcolor.h>                        // Color class
#include <qtimer.h>                        // Timer class
#include <qpixmap.h>                       // Pixmap
#include <qrect.h>                         // Rectangles
#include "TeDFrame.h"
#include "TeD.h"


// Cursor constructor

TextCursor::TextCursor(QWidget* parent, const char* name)
              : QObject (parent, name)
{

   extern int FontHeight;

   // Sets color and initial rectangle of the cursor
   color = 0;
   rcCursor.setRect(-1, -1, 2, FontHeight);

   // Create pixmap
   pm = new QPixmap (2, FontHeight);
   pm->fill (white);

   // Creates timer for blinking
   timer = new QTimer(this);
   connect( timer, SIGNAL(timeout()), SLOT(BlinkCursor ()));
}


// Cursor destructor

TextCursor::~TextCursor()
{
  delete pm;            // Cursor pixmap
  delete timer;         // Timer

}


// Cursor blinking managing (called by timer)

void TextCursor::BlinkCursor ()
{
  extern int FontHeight;

  if (color) {
    color = 0;
    cursor_restore();
    }

  else {
    color = 1;
    cursor_fill();
    }
}


// Function to switch off cursor

void TextCursor::cursor_off()
{

  // Stop timer
  timer->stop();

  // Clear cursor
  if (color) {
    color = 0;
    cursor_restore();
    }
}


// Function to switch on cursor

void TextCursor::cursor_on(struct cur cursor)
{

  extern int FontHeight;

  if (!color) {
    // Move cursor
    rcCursor.setRect (cursor.x, cursor.y, 2, FontHeight);

    // Save text below cursor
    cursor_save();

    // Sets cursor on
    color = 1;
    cursor_fill();

    // Update bottom window
    ((MainWindow *)parent())->UpdateBottomLine();

    // Start timer
    timer->start( 300 );
  }
}


// Function to fill cursor if needed

void TextCursor::cursor_fill()
{
  QPainter p;

  if (color)
    {
    if (p.begin(((MainWindow *)parent())->Text)) {

      // Make cursor black
      p.fillRect(rcCursor, black);
      p.end();
      }
    else
      color = 0;
    }
}


// Function to save text below cursor

void TextCursor::cursor_save()
{
  extern int FontHeight;

  bitBlt ( (QPaintDevice *) pm, 0, 0,
           (QPaintDevice *) ((MainWindow *)parent())->Text,
           rcCursor.x(), rcCursor.y(), 2, FontHeight, CopyROP);

}


// Function to restore text below cursor

void TextCursor::cursor_restore()
{
  extern int FontHeight;

  bitBlt ( (QPaintDevice *) ((MainWindow *)parent())->Text, rcCursor.x(),
             rcCursor.y(), (QPaintDevice *) pm, 0, 0, 2, FontHeight, CopyROP);

}
