/***************************************************************************
 *                                                                         *
 *    File:         TeDCursor.cpp                                          *
 *                                                                         *
 *    Description:  Cursor manager of TeD X-Window.                        *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Date:         29 Aug 1994                                            *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <qwidget.h>                   // Widgets
#include <qrect.h>                     // Rectangles
#include <qfontmet.h>                  // Font metrics
#include <qpainter.h>                  // Painting
#include "TeDFrame.h"
#include "TeD.h"

/*
** Function to compute the video coordinate of the cursor and refresh
** the visualization of the cursor. hwnd points to the main widget.
*/

void SetCursor(MainWindow *hwnd)
{
  extern struct cur cursor_pos;         // cursor video coordinates
  extern unsigned int cursor_row;       // cursor row
  extern unsigned int first_video_row;  // first video row
  extern unsigned int cursor_column;    // cursor column
  extern struct row *actual;            // pointer cursor row
  extern QRect rcClient;                // client window rectangle
  extern int LeftXScreen;               // left margin screen
  extern char fontname[];               // font to be used
  extern int size;                      // size of the font
  extern int FontAscent;               // font descent
  extern int FontHeight;                // font height
  extern int FontMaxWidth;              // font max width
  extern int WidthTable[];              // font width table

  unsigned int i;                       // counter


  // Compute y coordinate
  cursor_pos.y = rcClient.top() +
                 ((int)cursor_row - (int)first_video_row + 1L) * FontHeight -
                 FontAscent;

  // Compute x coordinate
  cursor_pos.x = rcClient.left() + FontMaxWidth - LeftXScreen;
  for (i = 0; i < cursor_column; ++i)
     cursor_pos.x += WidthTable[*(actual->text + i) & 0xFF];

  // Update cursor position
  hwnd->Cursor->cursor_on(cursor_pos);
}


/*
** Funzione per verificare se il cursore e' visibile all'interno della finestra
** client. Se il cursore non e' visibile, provvede a spostare la finestra in
** modo tale da renderlo visibile con il minimo spostamento possibile. Se
** e' visibile, provvede a visualizzarlo.
** N.B. Il contatore last_video_row non viene aggiornato correttamente.
*/

void ValidateCursor(MainWindow *hwnd)
{
  extern unsigned int first_video_row;      // first video row
  extern unsigned int last_video_row;       // last video row
  extern unsigned int first_video_line;     // first video line
  extern struct row *video;                 // pointer video line
  extern unsigned int cursor_row;           // cursor row
  extern unsigned int cursor_line;          // cursor line
  extern struct row *actual;                // pointer cursor row
  extern unsigned int cursor_column;        // cursor column
  extern int FontHeight;                    // font height
  extern int FontMaxWidth;                  // font max width
  extern int WidthTable[];                  // font width table
  extern QRect rcClient;                    // client widndow rectangle
  extern int LeftXScreen;                   // left screen margin

  int xscroll;
  int yscroll = 0;
  int i;

  // Check y coordinate
  if ((yscroll = (int)first_video_row - (int)cursor_row) > 0) {
    /* il cursore si trova al di sopra della prima riga visualizzata */
    first_video_row = cursor_row;
    first_video_line = cursor_line;
    video = actual;
    }
  else if ((yscroll = (int)last_video_row - (int)cursor_row) < 0) {
    /* il cursore si trova al di sotto dell'ultima riga visualizzata */
    xscroll = (int)last_video_row - (int)first_video_row;
    first_video_row = cursor_row;
    first_video_line = cursor_line;
    video = actual;
    for (i = 0; i < xscroll; ++i) {
      --first_video_row;
      video = video->prev;
      if (video->carriage != cont) --first_video_line;
      }
    }
  else
    yscroll = 0;

  yscroll *= FontHeight;


  // Check x coordinate

  xscroll = 0;
  for (i = 0; i < (int) cursor_column; ++i)
    xscroll += WidthTable[*(actual->text + i)  & 0xFF];
  if (xscroll < LeftXScreen) {
    /* il cursore si trova a sinistra della finestra client */
    i = xscroll;
    xscroll = LeftXScreen - xscroll;
    LeftXScreen = i;
    }
  else if ((xscroll += FontMaxWidth) > (i = LeftXScreen +
           rcClient.right() - rcClient.left() - FontMaxWidth)) {
    /* il cursore si trova a destra della finestra client */
    xscroll = i - xscroll;
    LeftXScreen -= xscroll;
    }
  else {
    /* il cursore si trova all'interno delle colonne visualizzate */
    xscroll = 0;
    }

  /* Scorrimento finestra client */

  if (xscroll != 0 || yscroll != 0) {
    i = rcClient.height();

    if (yscroll > i || yscroll < -i)
      hwnd->Text->repaint();

    else
      {
      if (xscroll)
         Scroll(hwnd->Text,xscroll,1);
      if (yscroll)
         Scroll(hwnd->Text,yscroll,0);
      }
  }
  SetCursor(hwnd);
}



/*
** Funzione per verificare se il cursore e' visibile all'interno della finestra
** client. Se il cursore non e' visibile, provvede a spostare la finestra in
** modo tale da renderlo visibile con il minimo spostamento possibile nella
** direzione x e centrandolo nella direzione y. Se e' visibile, provvede
** a visualizzarlo.
** N.B. Il contatore last_video_row non viene aggiornato correttamente.
*/

void CentrateCursor(MainWindow *hwnd)
{
  extern unsigned int first_video_row;     // first video row
  extern unsigned int last_video_row;      // last video row
  extern unsigned int first_video_line;    // first video line
  extern struct row *video;                // pointer video row
  extern unsigned int cursor_row;          // cursor row
  extern unsigned int cursor_line;         // cursor line
  extern struct row *actual;               // pointer cursor row
  extern unsigned int cursor_column;       // cursor column
  extern int FontHeight;                   // font height
  extern int FontMaxWidth;                 // font max width
  extern int WidthTable[];                 // font width table
  extern QRect rcClient;                   // client window rectangle
  extern int LeftXScreen;                  // left margin screen

  int xscroll;
  int yscroll = 0;
  int i;

  // Check y coordinate

  if ((yscroll = (int)first_video_row - (int)cursor_row) > 0) {
    /* il cursore si trova al di sopra della prima riga visualizzata */
    xscroll = ((int)last_video_row - (int)first_video_row)/2;
    first_video_row = cursor_row;
    first_video_line = cursor_line;
    video = actual;
    i = 0;
    while (i < xscroll && video->prev) {
      --first_video_row;
      ++yscroll;
      video = video->prev;
      if (video->carriage != cont) --first_video_line;
      ++i;
      }
    }
  else if ((yscroll = (int)last_video_row - (int)cursor_row) < 0) {
    /* il cursore si trova al di sotto dell'ultima riga visualizzata */
    xscroll = ((int)last_video_row - (int)first_video_row)/2;
    yscroll = (int)first_video_row - (int)cursor_row;
    first_video_row = cursor_row;
    first_video_line = cursor_line;
    video = actual;
    for (i = 0; i < xscroll; ++i) {
      --first_video_row;
      ++yscroll;
      video = video->prev;
      if (video->carriage != cont) --first_video_line;
      }
    }
  else
    yscroll = 0;

  yscroll *= FontHeight;


  // Check x coordinate

  xscroll = 0;
  for (i = 0; i < (int) cursor_column; ++i)
    xscroll += WidthTable[*(actual->text + i) & 0xFF]; // compute x coordinate
  if (xscroll < LeftXScreen) {
    /* il cursore si trova a sinistra della finestra client */
    i = xscroll;
    xscroll = LeftXScreen - xscroll;
    LeftXScreen = i;
    }
  else if ((xscroll += FontMaxWidth) > (i = LeftXScreen +
           rcClient.right() - rcClient.left() - FontMaxWidth)) {
    /* il cursore si trova a destra della finestra client */
    xscroll = i - xscroll;
    LeftXScreen -= xscroll;
    }
  else {
    /* il cursore si trova all'interno delle colonne visualizzate */
    xscroll = 0;
    }

  /* Scorrimento finestra client */

  if (xscroll != 0 || yscroll != 0) {
    i = rcClient.height();

    if (yscroll > i || yscroll < -i)
      hwnd->Text->repaint();

    else
      {
      if (yscroll)
         Scroll(hwnd->Text,yscroll,0);
      if (xscroll)
         Scroll(hwnd->Text,xscroll,1);
      }

  }

  SetCursor(hwnd);
}


/*
** Funzione di spostamento del cursore di una posizione a sinistra.
*/

void l_key(void)
{
  extern struct row *actual;
  extern unsigned int cursor_row;
  extern unsigned int cursor_line;
  extern unsigned int cursor_column;

  if (cursor_column)
    --cursor_column;

  else if (actual->prev)
    {
      actual = actual->prev;
      cursor_column = strlen(actual->text);
      --cursor_row;
      if (actual->carriage != cont) --cursor_line;
    }
}


/*
** Funzione di spostamento del cursore di una posizione a destra.
*/

void r_key(void)
{
  extern struct row *actual;
  extern unsigned int cursor_row;
  extern unsigned int cursor_line;
  extern unsigned int cursor_column;


  if (cursor_column < strlen(actual->text))
    ++cursor_column;

  else if (actual->next)
    {
      if (actual->carriage != cont) ++cursor_line;
      actual = actual->next;
      cursor_column = 0;
      ++cursor_row;
    }
}


/*
** Funzione di spostamento del cursore di una posizione in alto.
*/

void up_key(void)
{
  extern struct row *actual;
  extern unsigned int cursor_row;
  extern unsigned int cursor_line;
  extern unsigned int cursor_column;


  if (actual->prev)
    {
    actual = actual->prev;
    if (actual->carriage != cont) --cursor_line;
    --cursor_row;
    if (cursor_column > strlen(actual->text))
       cursor_column = strlen(actual->text);
    }
}


/*
** Funzione di spostamento del cursore di una posizione in basso.
*/

void dn_key(void)
{
  extern struct row *actual;
  extern unsigned int cursor_row;
  extern unsigned int cursor_line;
  extern unsigned int cursor_column;

  if (actual->next)
   {
    if (actual->carriage != cont) ++cursor_line;
    actual = actual->next;
    ++cursor_row;
    if (cursor_column > strlen(actual->text))
      cursor_column = strlen(actual->text);
   }
}


/*
** Funzione di spostamento del cursore all'inizio della riga.
*/

void hm_key(void)
{
  extern unsigned int cursor_column;
  cursor_column = 0;
}


/*
** Funzione di spostamento del cursore alla fine della riga.
*/

void end_key(void)
{
  extern struct row *actual;
  extern unsigned int cursor_column;

  cursor_column = strlen(actual->text);
}


/*
** Funzione di arretramento del cursore di una pagina.
*/

void pup_key(void)
{
  extern struct row *actual;
  extern unsigned int cursor_row;
  extern unsigned int cursor_line;
  extern unsigned int cursor_column;
  extern unsigned int first_video_row;
  extern unsigned int last_video_row;

  int scroll;
  int i;

  scroll = (int)last_video_row - (int)first_video_row + 1L;
  i = 0;
  while (i < scroll && actual->prev) {
     actual = actual->prev;
     if (actual->carriage != cont) --cursor_line;
     --cursor_row;
     ++i;
     }
  if (cursor_column > strlen(actual->text))
     cursor_column = strlen(actual->text);
}


/*
** Funzione di avanzamento del cursore di una pagina.
*/

void pdn_key()
{
  extern struct row *actual;
  extern unsigned int cursor_row;
  extern unsigned int cursor_line;
  extern unsigned int cursor_column;
  extern unsigned int first_video_row;
  extern unsigned int last_video_row;

  int scroll;
  int i;

  scroll = (int)last_video_row - (int)first_video_row + 1L;
  i = 0;
  while (i < scroll && actual->next) {
     if (actual->carriage != cont) ++cursor_line;
     actual = actual->next;
     ++cursor_row;
     ++i;
     }
  if (cursor_column > strlen(actual->text))
     cursor_column = strlen(actual->text);
}


/*
** Funzione di posizionamento del cursore all'inizio del testo.
*/

void chm_key(void)
{
  extern struct row *actual;
  extern struct row *first;
  extern unsigned int cursor_row;
  extern unsigned int cursor_line;
  extern unsigned int cursor_column;

  actual = first;
  cursor_row = 0;
  cursor_line = 1;
  cursor_column = 0;
}


/*
** Funzione di spostamento del cursore alla fine del testo.
*/

void cen_key(void)
{
  extern struct row *actual;
  extern struct row *last;
  extern unsigned int t_len;
  extern unsigned int t_line;
  extern unsigned int cursor_row;
  extern unsigned int cursor_line;
  extern unsigned int cursor_column;

  actual = last;
  cursor_row = t_len;
  cursor_line = t_line;
  cursor_column = strlen(actual->text);
}


/*
** Funzione di avanzamento del cursore all'inizio della parola
** seguente.
*/

void cr_key(void)
{
  extern struct row *actual;
  extern struct row *last;
  extern unsigned int cursor_column;
  int lrl;
  enum ret rc;

  /* Calcolo lunghezza ultima riga testo */

  lrl = strlen(last->text);

  /* Posizionamento al prossimo spazio o inizio riga */

  while ( *(actual->text + cursor_column) != 32)
    {
      r_key();

      /* Controllo fine file */
      if (actual == last && cursor_column == lrl) return;

      /* Controllo inizio nuova riga */
      if (cursor_column == 0 && (actual->prev ? actual->prev->carriage:
          hard) != cont) break;
    }

  /* Posizionamento al prossimo carattere diverso dallo spazio */

  while (*(actual->text + cursor_column) == 32 ||
         *(actual->text + cursor_column) == (char)NULL)
    {
      r_key();

      /* Controllo fine file */
      if (actual == last && cursor_column == lrl) return;
    }
}


/*
** Funzione di arretramento del cursore all'inizio della parola precedente.
*/

void cl_key(void)
{
  extern struct row *first;
  extern struct row *actual;
  extern unsigned int cursor_column;

  /* Arretra fino al primo carattere diverso dallo spazio */

  do
    {
      l_key();
      if (actual == first && cursor_column == 0) return;
    } while (*(actual->text + cursor_column) == 32 ||
      *(actual->text + cursor_column) == (char)NULL);

  /* Arretra fino all'inizio della parola */

  while (!((cursor_column == 0 && (actual->prev ? actual->prev->carriage :
            hard) != cont) ||
            (cursor_column > 0 && *(actual->text + (cursor_column ?
            cursor_column - 1: 0)) == 32)))
    {
      l_key();
      if (actual == first && cursor_column == 0) return;
    }
}

/*
** Funzione per posizionare il cursore sul carattere puntato dal mouse.
*/

void CursorAt(int x, int y)
{
  extern struct row *actual;
  extern unsigned int cursor_row;
  extern unsigned int cursor_line;
  extern unsigned int cursor_column;

  GetCharAt(x, y, &cursor_row, &cursor_line, &cursor_column, &actual);
  if (cursor_column > strlen(actual->text))
     cursor_column = strlen(actual->text);

}

/*
** Funzione per restituire la riga, la linea, la colonna e il
** puntatore della riga del carattere puntato dal mouse.
** <x> e <y> contengono la coordinata del mouse. Le coordinate del carattere
** vengono scritte nelle variabili puntate da <row>, <col> e <pointer>.
*/

void GetCharAt(int x, int y, unsigned int *prow, unsigned int *pline,
               unsigned int *pcol, struct row **pointer)
{
  extern QRect rcClient;
  extern int LeftXScreen;
  extern int FontAscent;
  extern int FontHeight;
  extern int FontMaxWidth;
  extern int WidthTable[];
  extern unsigned int first_video_row;
  extern unsigned int first_video_line;
  extern struct row *video;
  int i,n;
  char fOut = FALSE;

  /* Calcolo linea cursore */

  n = (y - rcClient.top() + FontAscent)/FontHeight;
  if (n <= 0 && video->prev) {
     *prow = first_video_row -1;
     *pointer = video->prev;
     *pline = first_video_line;
     if ((*pointer)->carriage != cont) --*pline;
     }
  else {
     *prow = first_video_row;
     *pline = first_video_line;
     *pointer = video;
     for (i = 1; i < n; ++i) {
        if ((*pointer)->next == NULL) {
          /* superata l'ultima riga del file */
          if (i < n) fOut = TRUE;
          break;
          }
        ++*prow;
        if ((*pointer)->carriage != cont) ++*pline;
        *pointer = (*pointer)->next;
        }
     }

  /* Calcolo colonna cursore */

  n = strlen((*pointer)->text);
  if (fOut) {                      /* Cursore alla fine del file */
     *pcol = n + 1;
     return;
     }
  x += (int)LeftXScreen - rcClient.left() - FontMaxWidth;
  i = 0;
  while (x + x > WidthTable[*((*pointer)->text + i) & 0xFF]) {
     x -= WidthTable[*((*pointer)->text + i) & 0xFF];
     ++i;
     if (i >= n) break;
     }
  *pcol = i;
}


/*
** Funzione di aggiornamento dei puntatori della posizione del cursore.
** <rel> e' il numero relativo di colonne di cui deve essere
** spostato il cursore. La riga su cui viene effettuata l'operazione
** e' quella indirizzata dal puntatore actual.
*/

void mov_cur(int rel)
{
  extern struct row *actual;             /* puntatore globale riga attuale */
  extern unsigned int cursor_row;        /* numero riga testo cursore      */
  extern unsigned int cursor_line;       /* numero linea testo cursore     */
  extern unsigned int cursor_column;     /* numero colonna cursore         */

  if (rel >= 0 && cursor_column > rel)
    {
      ++cursor_row;
      cursor_column -= rel + 1;
      if (actual->carriage == cont)
        ++cursor_column;
      else
        ++cursor_line;
      actual = actual->next;    /* Aggiorna la nuova riga */
    }
  else if (rel < 0)
    {
      if (cursor_column < (unsigned int)(-rel))
        {
          cursor_column += (strlen(actual->text) + 1);
          cursor_column -= (unsigned int)(-rel);
        }
      else
        {
          if (actual->carriage != cont)
             ++cursor_line;
          ++cursor_row;
          actual = actual->next;                /* Aggiorna la riga attuale */
          cursor_column -= (unsigned int)(-rel);
        }
    }
}

/*
** Funzione di cancellazione di un carattere alla posizione del cursore.
** Aggiorna lo schermo e la posizione del cursore.
*/

void del_key(MainWindow *hwnd)
{
  extern struct row *actual;
  extern struct row *first;
  extern unsigned int cursor_row;
  extern unsigned int cursor_column;
  extern unsigned int autosave;
  int i,len;

  if (*first->text || first->next) /* e' presente testo che sara' cancellato */
    modify(hwnd);

  if (cursor_column < (len = strlen(actual->text))) {

     /* Cancellazione nel mezzo */
      for (i = cursor_column; i < len; ++i)
         *(actual->text + i) = *(actual->text + i + 1);
      mov_cur(justfy(hwnd,cursor_column,cursor_row,(char *)NULL));
      }

  else if ((actual->carriage == hard && actual->next) || actual->carriage == soft){

    /* Unire al paragrafo seguente */
      actual->carriage = cont;
      mov_cur(justfy(hwnd,cursor_column,cursor_row,(char *)NULL));
      }

  else if (actual->carriage == cont) {
    /* Cancellare primo carattere riga seguente */

      r_key();
      len = strlen(actual->text);
      for (i = 0; i < len; ++i)
         *(actual->text + i) = *(actual->text + i + 1);
      mov_cur(justfy(hwnd,0,cursor_row,(char *)NULL));
    }
  ValidateCursor(hwnd);
}


/*
** Funzione di cancellazione del carattere con arretramento del cursore.
** Aggiorna lo schermo e il cursore.
*/

void bk_key(MainWindow *hwnd)
{
  extern struct row *first;
  extern struct row *actual;
  extern unsigned int cursor_column;

  if ( !(cursor_column == 0 && actual == first) )
    {
      l_key();
      if (actual->carriage == cont && cursor_column == ROWLEN)
        l_key();
      del_key(hwnd);
      return;
    }

  ValidateCursor(hwnd);
}