/***************************************************************************
 *                                                                         *
 *    File:         TeDFill.cpp                                            *
 *                                                                         *
 *    Description:  Paragraph filling of X-Window TeD.                     *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Date:         29 Aug 1994                                            *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "TeDFrame.h"
#include "TeD.h"


/*
** Funzione per riempire il buffer temporaneo puntato da <temp>.
** Il puntatore <source> punta alla prima riga sorgente da
** cui va estratto il testo. La variabile <n> punta ad un intero
** in cui viene scritta la posizione dell'ultima riga estratta nella
** stringa temporanea. Il puntatore <string> punta alla stringa formattata
** sorgente. <len> contiene la lunghezza della stringa string.
** <*pos> la posizione di inizio di estrazione. Il puntatore <phase>
** punta ad una variabile intera che contiene la fase attuale di composizione
** della riga (0 = riga precedente, 1 = inizio riga attuale, 2 = estrazione
** da stringa formattata, 3 = fine riga attuale, 4,5,... = formattazione
** righe successive. La funzione restituisce il puntatore alla riga
** successiva da cui va estratto il testo per riempire la stringa temporanea.
*/

struct row *fill(struct row *source,char *temp,int *n,char *string,
                 unsigned int len,unsigned int *pos,unsigned int *phase)
{
  static enum ret car;        /* ritorno carrello riga estratta */
  static char save_text[ROWLEN + 2];  /* salvataggio testo prima riga */
  extern unsigned int cursor_column;  /* colonna cursore  */

  /* Estrae le righe fino ad almeno ROWLEN + 1 caratteri */
  
  while ((source || *phase < 4) && strlen(temp) < ROWLEN + 1 &&
          !(*phase == 2 && car == hard && *temp != (char)NULL)) {
    *n = strlen(temp);                  /* Posizione ultima riga estratta */
    switch(*phase){
      case 0:
        car = extract(source,temp);     /* Estrazione riga */
        ++*phase;                       /* Passa alla fase successiva */
        source = source->next;          /* Avanzamento riga successiva */
        break;
      case 1:
        car = soft;                       /* Evita arresto estrazione */
        if (extract(source,temp) == hard) /* Fine paragrafo */
          source = (struct row *)NULL;    /* Invalida puntatore */
        else
          source = source->next;          /* Va alla riga successiva */
        ++*phase;                         /* Passa alla fase successiva */
        strcpy(save_text,temp + *n);      /* Salvataggio testo */
        *(temp + *n + cursor_column) = (char)NULL; /* Troncamento riga */
        break;
      case 2:
        car = ReadFormattedString(string,temp + *n,pos,len);
        if (*pos > len)                 /* Fine stringa inserimento */
          ++*phase;                     /* Passa alla fase successiva */
        else if (car == hard)           /* Esce per accettare hard return */
          return(source);               /* in assenza di testo */
        break;
      case 3:
        strcpy(temp + *n,save_text + cursor_column); /* Copia fine riga */
        ++*phase;                       /* Passa alla fase successiva */
        break;
      default:
        car = extract(source,temp);     /* Estrazione riga */
        ++*phase;                       /* Passa alla fase successiva */
        if (car == hard)                /* Fine paragrafo */
          source = (struct row *)NULL;  /* Invalida puntatore */
        else
          source = source->next;        /* Va alla riga successiva */
        break;
      }
    }
   return(source);
}

/*
** Funzione per scrivere la linea di testo <dest> con il contenuto
** del buffer temporaneo <temp>. La stringa cui punta <temp> deve
** avere una lunghezza di almeno ROWLEN + 1 caratteri. Dopo il
** riempimento della riga la stringa cui punta <temp> viene
** aggiornata. <ndest> contiene il numero della riga puntata da <dest>.
** La funzione aggiorna il numero di linee del testo ed il contatore
** della prima linea video.
** La funzione ritorna il numero di caratteri scritti.
*/

int writeline(struct row *dest,unsigned int ndest,char *temp)
{
  extern unsigned int t_line;       /* contatore globale numero linee testo */
  extern unsigned int first_video_row; /* contatore prima riga video  */
  extern unsigned int first_video_line;/* contatore prima linea video */
  int i,j;                             /* variabili temporanee */

  if ((j = strlen(temp)) <= ROWLEN) {  /* La stringa sta tutta sulla riga */
    strcpy(dest->text,temp);           /* copia stringa sulla riga     */
    if (dest->carriage == cont) {      /* aggiorna contatori linee */
      ++t_line;                        /* aumenta numero linee del testo */
      if (first_video_row > ndest)
        ++first_video_line;            /* aumenta numero prima riga video */
      }
    dest->carriage = hard;             /* termina riga con hard return */
    *temp = (char) NULL;               /* annulla la stringa temporanea */
    return(j);                         /* ritorna */
  }

  /* La stringa non sta tutta sulla riga */

  for (i = ROWLEN; i > -1; --i)
    if(*(temp + i) == 32) break;   /* spezza la riga alla lunghezza massima */

  if (i != -1) {                    /* Riga con soft return */
    *(temp + i) = (char) NULL;      /* scrive carattere di fine stringa  */
    strcpy(dest->text,temp);        /* copia parzialmente la stringa     */
    if (dest->carriage == cont) {
      ++t_line;                     /* aumenta numero linee del testo */
      if (first_video_row > ndest)
        ++first_video_line;         /* aumenta numero prima riga video */
      }
    dest->carriage = soft;          /* scrive soft return */
    memmove(temp,temp + i + 1,(unsigned int)(j - i));  /* aggiorna stringa temporanea */
    return(i + 1);                  /* ritorna */
    }

   /* Riga con continuazione */

   strncpy(dest->text,temp,(unsigned int) ROWLEN); /* copia la stringa */
   *(dest->text + ROWLEN) = (char) NULL;           /* termina la riga  */
   if (dest->carriage != cont) {
     --t_line;                  /* decrementa numero linee del testo */
     if (first_video_row > ndest)
       --first_video_line;      /* decrementa numero prima riga video */
     }
   dest->carriage = cont;       /* scrive continuation mark */

   /* aggiorna stringa temporanea */
   memmove(temp,temp + ROWLEN,(unsigned int)(j - ROWLEN + 1));
   return(ROWLEN);              /* ritorna */
}