/***************************************************************************
 *                                                                         *
 *    File:         TeDFind.cpp                                            *
 *                                                                         *
 *    Description:  Search manager of X-Window TeD                         *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Date:         1 Jan 1996                                             *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <qlabel.h>                      // Labels
#include <qdialog.h>                     // Dialogs
#include <qframe.h>                      // Frames
#include <qpushbt.h>                     // Push buttons
#include <qchkbox.h>                     // Check boxes
#include <qradiobt.h>                    // Radio buttons
#include <qbttngrp.h>                    // Button groups
#include <qlined.h>                      // Line edit
#include "TeDFrame.h"
#include "TeD.h"



// Find window constructor

WinFind::WinFind(QWidget* parent, const char* name, WFlags f)
        : QDialog (0, name, TRUE, f)
{

  // Save parent
  hwnd = parent;

  // Sets frame & separator
  frame = new QFrame(this);
  separator = new QFrame(this);
  frame->setFrameStyle( QFrame::Panel | QFrame::Raised );
  separator->setFrameStyle( QFrame::HLine | QFrame::Sunken );

  // Sets label
  Mlabel = new QLabel(this);
  Mlabel->setText("Find: ");
  Mlabel->setAlignment(AlignCenter);
  Mlabel->setAutoResize(TRUE);
  
  // Sets line edit
  line = new QLineEdit(this);
  line->setMaxLength(ROWLEN);                  // Max ROWLEN characters
  line->setText(((MainWindow *) hwnd)->ftext); // Copy text

  // Sets buttons
  Ok = new QPushButton(this);
  Cancel = new QPushButton(this);
  case_sens = new QCheckBox(this);
  dirbut = new QButtonGroup("Direction", this);
  rb[0] = new QRadioButton(dirbut);
  rb[1] = new QRadioButton(dirbut);

  Ok->setText("Ok");
  Cancel->setText("Cancel");
  case_sens->setText("Case sensitive");
  rb[0]->setText("Up");
  rb[1]->setText("Down");

  // Connect button signals
  connect(Ok, SIGNAL(clicked()), SLOT(ok_clicked()));
  connect(Cancel, SIGNAL(clicked()), SLOT(cancel_clicked()));
  connect(case_sens, SIGNAL(clicked()), SLOT(case_pressed()));
  connect(dirbut, SIGNAL(clicked(int)), SLOT(direction(int)));

  // Connect find signal
  connect (this, SIGNAL(answer(find_text)), parent,
           SLOT(FindAnswer(find_text)));

  // Check down direction button
  rb[1]->setChecked(TRUE);

  // Check case sensitive button
  case_sens->setChecked(TRUE);

  // Focus to line editor
  line->setFocus();

  // Initialize data
  ft.case_sensitive = TRUE;
  ft.down_search = TRUE;

  // Resize window & show
  resize(300, 200);
  show();
}

// Find window destructor

WinFind::~WinFind()
{
  // Free storage

  delete Mlabel;                 // Message box label
  delete line;                   // Line edit
  delete case_sens;              // Case sensitive check box
  delete[] rb;                   // Radio buttons
  delete dirbut;                 // Direction button group
  delete Ok;                     // Ok button
  delete Cancel;                 // Cancel button
  delete separator;              // Separator line
  delete frame;                  // Frame
}


// Find window resize event

void WinFind::resizeEvent( QResizeEvent* )
{
  const int space = 20;
  const int sspace = 10;
  const int bspace = 30;
  const int bbspace = 50;
  const int hei = 25;
  int blen = (this->width() - 2 * bspace - bbspace)/2;

  // Sets frame geometry
  frame->setGeometry(0, 0, this->width(), this->height());

  // Sets separator geometry
  separator->setGeometry(0, this->height() - bspace - hei, this->width(), 4);  

  // Sets label geometry
  Mlabel->setGeometry(sspace, sspace, 50, hei);

  // Sets line edit geometry
  line->setGeometry(space + 50, sspace, this->width() - space - bspace - 50, 
                    hei);

  // Sets case sensitive geometry
  case_sens->setGeometry(space, bspace + hei, 120, hei);

  // Sets button group geometry
  dirbut->setGeometry(this->width() - 100 - space, bspace + hei, 100, 3 * hei);
  rb[0]->setGeometry(sspace, space, 50, hei);
  rb[1]->setGeometry(sspace, space + hei, 50, hei);

  // Sets ok cancel buttons geometry
  Ok->setGeometry (bspace, this->height() - sspace - hei, blen, hei);
  Cancel->setGeometry (this->width() - bspace - blen,
                       this->height() - sspace - hei, blen, hei);

}

// Ok button pressed

void WinFind::ok_clicked()
{
  extern unsigned int cursor_row;
  extern unsigned int cursor_line;
  extern unsigned int cursor_column;
  extern struct row *actual;

  // Initialize search structure
  ft.start_row = cursor_row;
  ft.start_line = cursor_line;
  ft.start_column = cursor_column;
  ft.start_pointer = actual;
  strcpy(ft.text,line->text());

  // Copy find text to main application buffer
  strcpy(((MainWindow *) hwnd)->ftext, line->text());

  // Call search routine
  if (search_string(&ft))
    emit answer(ft);
  else
    ((MainWindow *)hwnd)->MessageBox("Find result: String not found", MB_OK);
}


// Cancel button pressed

void WinFind::cancel_clicked()
{

  // Close window
  close (FALSE);
}


// Case sensitive button pressed
void WinFind::case_pressed ()
{

  ft.case_sensitive = case_sens->isChecked();

}

// Direction changed

void WinFind::direction (int i)
{

  ft.down_search = i;

}


// Main Window find SLOT processor

void MainWindow::FindAnswer(find_text ft)
{
  extern unsigned int cursor_row;
  extern unsigned int cursor_line;
  extern unsigned int cursor_column;
  extern struct row *actual;
  extern unsigned int mark_row_end;
  extern unsigned int mark_line_end;
  extern unsigned int mark_column_end;
  extern struct row *mark_pointer_end;

  // switch off cursor
  this->Cursor->cursor_off();

  // deselect text
  Deselect(this);

  // Position cursor and selection
  cursor_row = ft.start_row;
  cursor_line = ft.start_line;
  cursor_column = ft.start_column;
  actual = ft.start_pointer;
  mark_row_end = ft.stop_row;
  mark_line_end = ft.stop_line;
  mark_column_end = ft.stop_column;
  mark_pointer_end = ft.stop_pointer;

  // Select found text
  Select(this);
  
  // Centrate cursor
  CentrateCursor(this);
}
