/***************************************************************************
******************************** TeDFrame.cpp ******************************
****************************************************************************/

// TeD system Frame declaration

#include <qwindow.h>                 // Window
#include <qscrbar.h>                 // Scroll bar
#include <qevent.h>                  // Events
#include <qpoint.h>                  // Points
#include <qrect.h> 	             // Rectangles
#include <string.h>                  // String functions
#include <stdlib.h>
#include "TeDFrame.h"
#include "TeD.h"

// Main window constructor

MainWindow::MainWindow(QWidget *parent, const char *name,WFlags f)
        : QWindow(parent,name,f)
{
    extern QSize WinSize;

    // Sets window name
    char Title[CCHMAXPATH+6] = "TeD - ";
    extern char szFullFile[CCHMAXPATH];
    strcat(Title,szFullFile);
    setCaption(Title);

    // Enables mouse tracking
    setMouseTracking(TRUE);

    // Gets font metrics
    SetFontMetrics();

    // Creates menu bar
    MenuCreate();

    // Create widget for displaying text
    Text = new TextWidget(this);

    // Create scrollbars and resizes window
    vscroll = new QScrollBar (QScrollBar::Vertical,this);
    hscroll = new QScrollBar (QScrollBar::Horizontal,this);

    // Create bottom line
    bottom = new QLabel(this);
    bottom->setFrameStyle( QFrame::Panel | QFrame::Sunken );
    bottom->setAlignment( AlignVCenter | AlignLeft );
    bottom->setFont (QFont ("lucidabright",14, QFont::Bold,TRUE));

    // Connect vertical scrollbar signals
    connect(vscroll,SIGNAL(valueChanged(int)),SLOT(SetVPosition(int)));
    connect(vscroll,SIGNAL(sliderPressed()),SLOT(VSliderPressed()));
    connect(vscroll,SIGNAL(sliderReleased()),SLOT(VSliderReleased()));

    // Connect horizontal scrollbar signals
    connect(hscroll,SIGNAL(valueChanged(int)),SLOT(SetHPosition(int)));
    connect(hscroll,SIGNAL(sliderPressed()),SLOT(HSliderPressed()));
    connect(hscroll,SIGNAL(sliderReleased()),SLOT(HSliderReleased()));

    // Initialize data for find / replace
    strcpy(ftext,"");
    strcpy(rtext,"");
    strcpy(stext,"");

    // Resize window
    resize(WinSize);
}


// Resizing event of the main window

void MainWindow::resizeEvent( QResizeEvent* )
{
    extern QRect rcClient;    // Client window rectangle

    // Resizes scrollbars
    int v_x,v_y,v_w,v_h;      // Geometry vertical scrollbar
    int h_x,h_y,h_w,h_h;      // Geometry horizontal scrollbar

    v_w = h_h = SCROLLW;                      // Width scrollbars
    h_w = this->width() - SCROLLW;            // Length horizontal scrollbar
    v_h = this->height() - menu->height()
          - h_h - BOTTOMW - SPACE - SPACE;    // Lenght vertical scrollbar
    v_x = this->width()- v_w + 1;             // x start position vertical
    v_y = menu->height();                     // y start position vertical
    h_x = 0;                                  // x start position horizontal
    h_y = this->height() - BOTTOMW 
          - SPACE - SPACE - h_h + 1;          // y start position horizontal

    vscroll->setGeometry(v_x,v_y,v_w,v_h);
    hscroll->setGeometry(h_x,h_y,h_w,h_h);

    // Resize bottom label
    bottom->setGeometry(SPACE, this->height() - SPACE - BOTTOMW + 1,
                        this->width() - SPACE - SPACE, BOTTOMW);

    // Save dimensions of the window
    rcClient.setTop(menu->height());
    rcClient.setBottom(h_y - 1);
    rcClient.setLeft(0);
    rcClient.setRight(v_x - 1);

    // Resizes text window
    Text->setGeometry(rcClient);
    rcClient.setBottom(rcClient.height() - 1);
    rcClient.setTop(0);
}
