/***************************************************************************
******************************** TeDFrame.h ********************************
****************************************************************************/

// TeD system Main class declaration

#include <qwindow.h>                 // Window
#include <qmenubar.h>                // Menu Bar
#include <qscrbar.h>                 // Scroll bar
#include <qevent.h>                  // Events
#include <qpainter.h>                // Painting
#include <qfont.h>                   // Fonts
#include <qdialog.h>                 // Dialog
#include <qlabel.h>                  // Labels
#include <qwidget.h>                 // Widgets
#include <qframe.h>                  // Frame
#include <qchkbox.h>                 // Check boxes
#include <qradiobt.h>                // Radio buttons
#include <qbttngrp.h>                // Button groups
#include <qtimer.h>                  // Timer
#include <qlined.h>                  // Line edit
#include <qpixmap.h>                 // Pixmap
#include <qrect.h>                   // Points
#include <qcombo.h>                  // Combo box
#include <unistd.h>                  // Read and write from pipe

#define ROWLEN 72                    // Last column to write
#define MAXBUFFER 1024               // Maximum lenght buffer from/to ispell

// Find text class

class find_text
{
public:

char text[ROWLEN+1];
char case_sensitive;
char down_search;
unsigned int start_row;
unsigned int start_line;
unsigned int start_column;
struct row *start_pointer;
unsigned int stop_row;
unsigned int stop_line;
unsigned int stop_column;
struct row *stop_pointer;
};


// Text cursor class

class TextCursor : public QObject
{
   Q_OBJECT
public:
   TextCursor(QWidget* parent=0, const char* name=0);
   ~TextCursor();
   QPixmap *pm;                            // Pixmap
   void cursor_on(struct cur cursor);
   void cursor_off();
   void cursor_fill();
   void cursor_restore();
   void cursor_save();

private:
   int color;                               // Color of the cursor
   QTimer *timer;                           // Timer
   QRect rcCursor;                          // Cursor rectangle

private slots:
   void BlinkCursor ();                     // Blinking cursor
};


// Text widget class

class TextWidget : public QWidget
{
   Q_OBJECT

public:
   TextWidget(QWidget* parent=0, const char* name=0, WFlags f=0);

protected:
    void paintEvent( QPaintEvent * );            // Paint event handler
    void mouseMoveEvent (QMouseEvent *);         // Mouse move event handler
    void mousePressEvent (QMouseEvent *);        // Mouse press event
    void mouseReleaseEvent (QMouseEvent *);      // Mouse release event
    void mouseDoubleClickEvent (QMouseEvent*);   // Mouse double click event

private:
   QTimer *timer;                                // Timer for scrolling
   int mousex, mousey;                           // Last mouse coordinates

public slots:
   void ScrollSelection();                       // Selection extender
};


// Message Box class

class WinMessageBox : public QDialog
{
   Q_OBJECT
public:
   WinMessageBox(char* text, int type, QWidget* parent=0, const char* name=0,
                 bool modal=FALSE,WFlags f=0);

   ~WinMessageBox();


signals:
   void answer(int);            // Answer

public slots:
   void yes_clicked();          // Yes pressed
   void no_clicked();           // No pressed
   void cancel_clicked();       // Cancel clicked

private:
   QLabel *Mlabel;                         // Message box label
   QPushButton *Yes;                       // Yes button
   QPushButton *No;                        // No button
   QPushButton *Cancel;                    // Cancel button
   QFrame *separator;                      // Separator line
   QFrame *frame;                          // Frame
   int Mtype;                              // Type of message box

protected:
   void resizeEvent( QResizeEvent* );      // Resize event handler
};


// Goto window class

class WinGoto : public QDialog
{
    Q_OBJECT

public:
   WinGoto(QWidget* parent=0, const char* name=0, WFlags f=0);

   ~WinGoto();

private:
   QLabel *Mlabel;                         // Message box label
   QLineEdit *line;                        // Line edit
   QPushButton *Ok;                        // Ok button
   QPushButton *Cancel;                    // Cancel button
   QFrame *separator;                      // Separator line
   QFrame *frame;                          // Frame
   QWidget *hwnd;                          // Parent pointer

protected:
   void resizeEvent( QResizeEvent* );      // Resize event handler

signals:
   void answer(int);                       // Answer signal

public slots:
   void ok_clicked();                      // Ok pressed
   void cancel_clicked();                  // Cancel clicked
};


// Autosave window class

class WinAutosave: public QDialog
{
    Q_OBJECT
public:
   WinAutosave(QWidget* parent=0, const char* name=0, WFlags f=0);
   ~WinAutosave();

private:
   QLabel *Mlabel1;                        // Message box label
   QLabel *Mlabel2;                        // Message box label
   QLineEdit *line;                        // Line edit
   QPushButton *Ok;                        // Ok button
   QPushButton *Cancel;                    // Cancel button
   QFrame *separator;                      // Separator line
   QFrame *frame;                          // Frame
   QWidget *hwnd;                          // Parent pointer

protected:
   void resizeEvent( QResizeEvent* );      // Resize event handler

public slots:
   void ok_clicked();                      // Ok pressed
   void cancel_clicked();                  // Cancel clicked
};


// Find window class

class WinFind: public QDialog
{
    Q_OBJECT

public:
   WinFind(QWidget* parent=0, const char* name=0, WFlags f=0);
   ~WinFind();
   find_text ft;                           // Find text structure

private:
   QLabel *Mlabel;                         // Message box label
   QLineEdit *line;                        // Line edit
   QCheckBox *case_sens;                   // Case sensitive check box
   QRadioButton *rb[2];                    // Radio buttons search direction
   QButtonGroup *dirbut;                   // Direction button group
   QPushButton *Ok;                        // Ok button
   QPushButton *Cancel;                    // Cancel button
   QFrame *separator;                      // Separator line
   QFrame *frame;                          // Frame
   QWidget *hwnd;                          // Parent pointer

protected:
   void resizeEvent( QResizeEvent* );      // Resize event handler

signals:
   void answer(find_text f);               // Answer signal

public slots:
   void ok_clicked();                      // Ok pressed
   void cancel_clicked();                  // Cancel clicked
   void direction(int);                    // Direction changed
   void case_pressed();                    // Case sensitive
};


// Substitute window class

class WinSubs: public QDialog
{
    Q_OBJECT

public:
   WinSubs(QWidget* parent=0, const char* name=0, WFlags f=0);
   ~WinSubs();
   find_text ft;                           // Find text structure

private:
   QTimer *timer;                          // Timer
   QLabel *Mlabel1;                        // Message box label
   QLabel *Mlabel2;                        // Message box label
   QLineEdit *line1;                       // Line edit
   QLineEdit *line2;                       // Line edit
   QCheckBox *case_sens;                   // Case sensitive check box
   QRadioButton *rb[2];                    // Radio buttons search direction
   QButtonGroup *dirbut;                   // Direction button group
   QPushButton *Ok;                        // Ok button
   QPushButton *Cancel;                    // Cancel button
   QPushButton *Yes;                       // Yes button
   QPushButton *All;                       // All button
   QPushButton *No;                        // No button
   QFrame *separator;                      // Separator line
   QFrame *frame;                          // Frame
   QWidget *hwnd;                          // Parent pointer
   char subs[ROWLEN+1];                    // Substitution text
   int nsubs;                              // Number of substitution
   void ReplaceResult();                   // Visualize replace results
   char fAll;                              // Substitute all occurrences

protected:
   void resizeEvent( QResizeEvent* );      // Resize event handler

signals:
   void answer(find_text f);               // Answer signal

public slots:
   void ok_clicked();                      // Ok pressed
   void cancel_clicked();                  // Cancel clicked
   void yes_clicked();                     // Yes clicked
   void no_clicked();                      // No clicked
   void all_clicked();                     // All clicked
   void direction(int);                    // Direction changed
   void case_pressed();                    // Case sensitive
};

// Spell-check window class

class WinSpell: public QDialog
{
    Q_OBJECT

public:
   WinSpell(QWidget* parent=0, const char* name=0, WFlags f=0);
   ~WinSpell();

private:
   QTimer *rtimer;                         // Timer to read
   QTimer *wtimer;                         // Timer to write
   QTimer *itimer;                         // Timer to parse
   QLabel *Mlabel;                         // Message box label
   QLabel *Mlabelp;                        // Message box label
   QLineEdit *line;                        // Line edit
   QComboBox *combo;                       // Combo box
   QPushButton *Replace;                   // Replace button
   QPushButton *Skip;                      // Skip button
   QPushButton *Accept;                    // Accept button
   QPushButton *Insert;                    // Insert button
   QPushButton *InsertLC;                  // Insert lower-case button
   QPushButton *Quit;                      // Quit button
   QFrame *separator;                      // Separator line
   QFrame *frame;                          // Frame
   QWidget *hwnd;                          // Parent pointer
   char read_buffer[MAXBUFFER];            // Reading buffer
   char write_buffer[MAXBUFFER];           // Writing buffer
   int command;                            // Command to be executed
   int spell_start;                        // Beginning of spell check
   unsigned int column_check;              // Column begin check
   unsigned int rowline_check;             // Number of row to check
   unsigned int line_check;                // Number of line to check
   struct row *row_check;                  // Pointer to row to check
   void spell_in();                        // Input parser
   void spell_out();                       // Output parser

protected:
   void resizeEvent( QResizeEvent* );      // Resize event handler

public slots:
   void read_ispell();                     // Reading from ispell process
   void write_ispell();                    // Writing to ispell process
   void replace_clicked();                 // Replace clicked
   void skip_clicked();                    // Skip clicked
   void accept_clicked();                  // Accept clicked
   void insert_clicked();                  // Insert clicked
   void insertlc_clicked();                // Insert lower-case clicked
   void start_check();                     // Spell checking routine
   void quit_clicked();                    // Quit clicked
   void combo_update(int);                 // Combo box update
};


// Product information window class

class WinProduct: public QDialog
{
    Q_OBJECT
public:
   WinProduct(QWidget* parent=0, const char* name=0, WFlags f=0);
   ~WinProduct();

private:
   QLabel *Mlabel1;                        // Message box label
   QLabel *Mlabel2;                        // Message box label
   QLabel *Mlabel3;                        // Message box label
   QLabel *Mlabel4;                        // Message box label
   QPushButton *Ok;                        // Ok button
   QFrame *separator;                      // Separator line
   QFrame *frame;                          // Frame

protected:
   void resizeEvent( QResizeEvent* );      // Resize event handler

public slots:
   void ok_clicked();                      // Ok pressed
};


// Main window class

class MainWindow : public QWindow
{
    Q_OBJECT
public:
    MainWindow(QWidget *parent=0, const char *name=0, WFlags f=0);
    TextCursor*  Cursor;                       // Text cursor
    TextWidget*  Text;                         // Text widget
    QScrollBar*  vscroll;                      // Vertical scroll bar
    QScrollBar*  hscroll;                      // Horizontal scroll bar
    int MessageBox(char* text,int type);       // Message box 
    void UpdateBottomLine(void);               // Update bottom line    
    char ftext[ROWLEN + 1];                    // Find text
    char rtext[ROWLEN + 1];                    // Text to replace
    char stext[ROWLEN + 1];                    // Replacing text

private:
    QMenuBar  *menu;                           // Menu bar
    QPopupMenu *FileMenu;                      // File menu
    QPopupMenu *EditMenu;                      // Edit menu
    QPopupMenu *OptionsMenu;                   // Options menu
    QPopupMenu *FontMenu;                      // Font menu
    QPopupMenu *FamilyMenu;                    // Font family menu
    QPopupMenu *SizeMenu;                      // Font size menu
    QPopupMenu *HelpMenu;                      // Help menu
    QLabel *bottom;                            // Bottom line label
    WinMessageBox *message;                    // Message box
    WinGoto *gotoline;                         // Goto window
    WinAutosave *asave;                        // Autosave window
    WinFind *find;                             // Find window
    WinSubs *subs;                             // Substitute window
    WinSpell *spell;                           // Spell-check window
    WinProduct *product;                       // Info window
    void SetFontMetrics(void);                 // Set font metrics information
    void MenuCreate(void);                     // Creates menu bar
    void NewFile(void);                        // New File function
    void OpenFile(void);                       // Open File function
    void SaveAsFile(void);                     // Save As function
    int answer;                                // Message box answer;

protected:
    void mouseMoveEvent (QMouseEvent *);       // Mouse move event handler
    void resizeEvent( QResizeEvent* );         // Resize event handler
    void keyPressEvent (QKeyEvent *);          // Keypress handler
    void closeEvent (QCloseEvent *);           // Close event

public slots:
    void menuActivated(int);                   // Menu bar manager
    void fileActivated(int);                   // File menu manager
    void editActivated(int);                   // Edit menu manager
    void optionsActivated(int);                // Options menu manager
    void familyActivated(int);                 // Font family activated
    void pSizeActivated(int);                  // Font size activated
    void helpActivated(int);                   // Help menu manager
    void SetVPosition(int);                    // Scroll to a given line
    void VSliderPressed();                     // Vertical slider pressed
    void VSliderReleased();                    // Vertical slider released
    void SetHPosition(int);                    // Scroll to a given column
    void HSliderPressed();                     // Horizontal slider pressed
    void HSliderReleased();                    // Horizontal slider released
    void MessageAnswer(int);                   // Answer of a message box
    void GotoAnswer(int);                      // Answer of the goto box
    void FindAnswer(find_text);                // Answer of the find box
};
