/***************************************************************************
 *                                                                         *
 *    File:         TeDGoto.cpp                                            *
 *                                                                         *
 *    Descrizione:  Gestione salto del sistema TeD X-Windows               *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Data:         29 Ago 1994                                            *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <qlabel.h>                      // Labels
#include <qdialog.h>                     // Dialogs
#include <qframe.h>                      // Frames
#include <qpushbt.h>                     // Push buttons
#include <qlined.h>                      // Line edit
#include "TeDFrame.h"
#include "TeD.h"


/*
** Funzione per spostarsi alla riga specificata dall'intero
** intero senza segno <jmp>. Sposta il cursore
** all'inizio della riga specificata.
*/

void go_to(unsigned int jmp)
{
  extern unsigned int cursor_row;
  extern unsigned int cursor_line;
  extern unsigned int cursor_column;
  extern struct row *actual;

  cursor_column = 0;                           /* ritorno a capo cursore */

  if (jmp <= cursor_line) {              /* spostamento in alto */
      while (actual->prev && (jmp < cursor_line || (((actual->prev) ?
            actual->prev->carriage : hard) == cont))) {
          actual = actual->prev;
          --cursor_row;
          if (actual->carriage != cont)
             --cursor_line;
          }
    }
  else                                          /* spostamento in basso */
    {
      while (jmp > cursor_line && actual->next) {
          if (actual->carriage != cont)
             ++cursor_line;
          actual = actual->next;
          ++cursor_row;
        }
    }
}


// Goto window constructor

WinGoto::WinGoto(QWidget* parent, const char* name, WFlags f)
        : QDialog (0, name, TRUE, f)
{

  // Save parent window
  hwnd = parent;

  // Sets frame & separator
  frame = new QFrame(this);
  separator = new QFrame(this);
  frame->setFrameStyle( QFrame::Panel | QFrame::Raised );
  separator->setFrameStyle( QFrame::HLine | QFrame::Sunken );

  // Sets label
  Mlabel = new QLabel(this);
  Mlabel->setText("Go to line: ");
  Mlabel->setAlignment(AlignCenter);
  Mlabel->setAutoResize(TRUE);
  
  // Sets line edit
  line = new QLineEdit(this);
  line->setMaxLength(32);                               // Max 32 characters

  // Sets buttons
  Ok = new QPushButton(this);
  Cancel = new QPushButton(this);
  Ok->setText("Ok");
  Cancel->setText("Cancel");

  // Connect button signals
  connect(Ok, SIGNAL(clicked()), SLOT(ok_clicked()));
  connect(Cancel, SIGNAL(clicked()), SLOT(cancel_clicked()));

  // Connect goto signal
  connect (this, SIGNAL(answer(int)), parent, SLOT(GotoAnswer(int)));

  // Focus to line editor
  line->setFocus();

  // Resize window & show
  resize(240, 104);
  show();
}

// Goto window destructor

WinGoto::~WinGoto()
{
  // Free storage

  delete Mlabel;                 // Message box label
  delete line;                   // Line edit
  delete Ok;                     // Ok button
  delete Cancel;                 // Cancel button
  delete separator;              // Separator line
  delete frame;                  // Frame
}


// Goto window resize event

void WinGoto::resizeEvent( QResizeEvent* )
{
  const int space = 20;
  const int sspace = 10;
  const int bspace = 30;
  const int hei = 25;
  int blen = (this->width() - 3 * bspace)/2;

  // Sets frame geometry
  frame->setGeometry(0, 0, this->width(), this->height());

  // Sets separator geometry
  separator->setGeometry(0, bspace + hei , this->width(), 4);  

  // Sets label geometry
  Mlabel->setGeometry(sspace, sspace, 80, hei);

  // Sets line edit geometry
  line->setGeometry(space + 80, sspace, this->width() - space - bspace - 80, 
                    hei);

  // Sets buttons geometry
  Ok->setGeometry (bspace, this->height() - sspace - hei, blen, hei);
  Cancel->setGeometry (this->width() - bspace - blen,
                       this->height() - sspace - hei, blen, hei);

}

// Ok button pressed

void WinGoto::ok_clicked()
{
  int go;

  // Gets jump line
  if (sscanf(line->text(), "%d", &go)) {
    if (go <= 0)
      go = 1;

    emit answer(go);
    close (FALSE);                    // Close window
  }

}


// Cancel button pressed

void WinGoto::cancel_clicked()
{
  // Switch on cursor
  SetCursor((MainWindow *)hwnd);

  // Close window
  close (FALSE);
}


// Main Window goto line SLOT processor

void MainWindow::GotoAnswer(int go)
{

  Deselect(this);
  go_to((unsigned int) go);
  CentrateCursor(this);

}