/***************************************************************************
 *                                                                         *
 *    File:         TeDJstfy.c                                             *
 *                                                                         *
 *    Descrizione:  Giustificazione paragrafi del sistema TeD OS/2         *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Data:         29 Ago 1994                                            *
 *                                                                         *
 ***************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "TeDFrame.h"
#include "TeD.h"

/*
** Funzione di giustificazione automatica delle righe con rinfresco
** del video. Viene elaborata la riga attuale del cursore.
** La funzione ritorna la lunghezza della riga giustificata se la riga
** precedente non e' stata modificata, in caso contrario il numero 
** cambiato di segno di caratteri aggiunti alla riga precedente.
** Imposta il puntatore alla riga attuale alla prima riga modificata.
** Aggiorna il contatore del numero della riga e della linea del
** cursore oltre a tutti i puntatori globali tranne il numero
** dell'ultima riga visualizzata e il contatore
** del numero della colonna del cursore. <hwnd> punta alla finestra
** client per l'aggiornamento video. <first_modified> indica la prima
** colonna modificata della riga attuale. <last_modified> >= cursor_row
** indica l'ultima riga modificata. <ins_line> punta ad una stringa
** formattata contenente il testo da inserire alla posizione del cursore.
*/

int justfy(MainWindow *hwnd, unsigned int first_modified,
           unsigned int last_modified, char *ins_line)
{

  extern struct row *actual;              /* puntatore posizione cursore */
  extern unsigned int cursor_row;         /* contatore riga cursore */
  extern unsigned int cursor_line;        /* contatore linea cursore */
  extern unsigned int first_video_row;    /* contatore prima riga video */
  extern unsigned int first_video_line;   /* contatore prima linea video */
  extern struct row *video;               /* puntatore prima riga video */

  int m;                 /* lunghezza riga precedente */
  int bufpos;            /* posizione ultima stringa letta nel buffer */
  unsigned int phase;    /* contatore fase della giustificazione */
  unsigned int comp;     /* fase di paragone */
  unsigned int len;      /* lunghezza stringa da inserire */
  unsigned int pos = 0;  /* posizione nella stringa da inserire */
  char line_cgd = FALSE; /* flag alterazione numero delle righe */
  char k = FALSE;        /* giustificazione riga precedente (se k = TRUE)  */
  struct row *source;    /* puntatore alla riga sorgente giustificazione */
  struct row *dest;      /* puntatore alla riga destinazione da giustificare */
  unsigned int ndest;    /* numero riga da giustificare */
  enum ret car;          /* ritorno carrello riga in elaborazione */
  enum ret prec;         /* ritorno carrello riga precedente */
  char temp[2*ROWLEN+3]=""; /* buffer di ingresso */

  source = dest = actual;               /* inizio giustificazione */
  ndest = cursor_row;                   /* numero della riga base */
  last_modified -= ndest;               /* normalizza last_modified */
  car = soft;                           /* inizializza ritorno carrello */

  if (ins_line) {                       /* bisogna inserire una stringa */
    phase = 0;                          /* comincia dalla fase 0 */
    len = strlen(ins_line);             /* lunghezza stringa */
    last_modified += 4;                 /* aggiorna last_modified */
    comp = 5;                           /* inizializza fase paragone */
    }
  else {                                /* non bisogna inserire una stringa */
    phase = 4;                          /* comincia dalla fase 4 */
    last_modified += 6;                 /* aggiorna last_modified */
    comp = 7;                           /* inizializza fase paragone */
    }

  if ((prec = (actual->prev ? actual->prev->carriage : hard)) != hard) {
    /* bisogna considerare anche la riga precedente */

    k = TRUE;          /* segnalazione inizio giustificazione dalla riga
                          precedente quella attuale */

    source = dest = actual->prev;
    --ndest;
    m=strlen(source->text);            /* lunghezza riga */
    source = fill(source,temp,&bufpos,ins_line,len,&pos,&phase);
    bufpos -= writeline(dest,ndest,temp);
    car = dest->carriage;
    if (m == strlen(dest->text) && prec == car)
       k = FALSE;         /* la riga precedente non e' cambiata */
    else {
      /* invalida rettangolo video della riga */
      VideoUpdate(ndest,dest->text,hwnd,(unsigned int) m);
      first_modified = 0;             /* indica modifica alla riga base */
      --cursor_row;                   /* indietreggia riga cursore */
      actual = dest;                  /* indietreggia puntatore cursore */
      if (prec == cont)
         m = -1;
      else {
        --cursor_line;
        m -=  strlen(actual->text);
        }
      }
    if (car != hard || phase < 4) {
      dest = dest->next;
      ++ndest;
      }
    }
  else {                   /* la riga precedente non e' stata modificata */
    if (phase == 0)
      ++phase;             /* comincia dalla fase 1 anziche' dalla 0 */
    else {
      --comp;              /* decrementa fase paragone */
      --last_modified;     /* rinormalizza ultima riga modificata */
      }         
    }

  /* Giustifica prima riga */

  if (car != hard || phase < 4) {
    prec = dest->carriage;              /* memorizza ritorno carrello riga */
    source = fill(source,temp,&bufpos,ins_line,len,&pos,&phase);
    bufpos -= writeline(dest,ndest,temp);         /* giustifica riga */
    if (first_modified > strlen(dest->text))
      /* calcola prima colonna modificata */
       first_modified = strlen(dest->text);
 
    VideoUpdate(ndest,dest->text,hwnd,
                first_modified);   /* invalida rettangolo video della riga */
    car = dest->carriage;          /* ritorno carrello riga giustificata */

    /* Giustifica righe successive */
    while (phase < 4 || (car != hard && !(comp > last_modified &&
          ((source && source == dest->next && *temp == (char) NULL) ||
          phase == comp && bufpos == 0)))) {

      source = fill(source,temp,&bufpos,ins_line,len,&pos,&phase);
      if (comp > phase) {
         dest = add(dest);            /* Aggiungi nuova riga se manca */
         line_cgd = TRUE;             /* Segnala aggiunta riga */
         if (first_video_row > ndest) {
           ++first_video_row;         /* Aggiorna contatori prima riga video */
           ++first_video_line;
           }
         }
       else {
         dest = dest->next;           /* Passa alla riga successiva */
         ++comp;                      /* Incrementa fase paragone */
         prec = dest->carriage;       /* Memorizza ritorno carrello */
         }
       ++ndest;
       bufpos -= writeline(dest,ndest,temp);/* giustifica riga */
       car = dest->carriage;                /* ritorno carrello nuova riga */
      }
    }
  /* Cancella eventuali righe in eccedenza */

  last_modified = k ? cursor_row + 2 : cursor_row + 1;
  if (car == hard) {
     while (prec != hard) {
       dest = dest->next;
       prec = dest->carriage;
       if (first_video_row > ndest) {
         --first_video_row;
         if (video == dest) {
            /* ovvero first_video_row = ndest dopo il decremento */

            /* la prima riga video e' la riga da cancellare */
            last_modified = first_video_row;   /* aggiorna tutto lo schermo */
            video = video->prev;               /* arretra puntatore video */
            --first_video_line;           /* decrementa numero linea video */
            }
         else if (prec != cont) --first_video_line; 
                                        /* decrementa numero linea video */
         }
       dest = erase(dest);                             /* cancella riga */
       line_cgd = TRUE;                /* segnala arretramento righe video */
       }
  }

  /* Aggiornamento schermo */

  if (line_cgd)
     DownUpdate((last_modified > ndest + 1 ? ndest + 1: last_modified),hwnd);
  else
     LinesUpdate(last_modified,ndest,hwnd);

  /* Calcolo lunghezza riga giustificata */

  if (k)
    return (m);

  return (strlen(actual->text));
}