/***************************************************************************
 *                                                                         *
 *    File:         TeDKeys.cpp                                            *
 *                                                                         *
 *    Description:  Keyboard manager of the system TeD X-Window            *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1995.                     *
 *                  All rights reserved.                                   *
 *                                                                         *
 *    Date:         29 Aug 1994                                            *
 *                                                                         *
 ***************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <qevent.h>                    // Events handler
#include <qkeycode.h>                  // Key codes
#include "TeDFrame.h"
#include "TeD.h"


/*
** Handler to receive key-press events
*/

void MainWindow::keyPressEvent (QKeyEvent *e) 

{
  extern char fInsert;
  extern char fmark;
  char temp[2];

  // Special keys insensitive to modifiers (CTRL - ALT - SHIFT)
  switch(e->key())
     {
     case Key_Return:                // Return key
     case Key_Enter:
       this->Cursor->cursor_off();   // Switch off cursor
       if (fmark) {
         temp[0] = (char)EOL;
         temp[1] = (char)NULL;
         DelMark(this,temp);         // Substitute highlighted area
         }
       else
         ent_key(this);
       
       e->accept();                  // Key accepted
       return;

     case Key_Backspace:
       this->Cursor->cursor_off();   // Switch off cursor
       if (fmark) {
         DelMark(this,"");             // Erase highlighted area
	 ValidateCursor(this);         // Validate cursor
	 this->Cursor->cursor_off();   // Switch off cursor
       }
       bk_key(this);
       e->accept();
       return;
     }

  // Special keys without modifiers (CTRL - ALT - SHIFT)
  if (!e->state()) {
    switch (e->key()) {

       case Key_Delete:                // Accelerator for DEL menu
         if (check_clear())
           editActivated(IDM_CLEAR);
       e->accept();
       return;

      case Key_Insert:                // Accelerator for Insert menu
        optionsActivated(IDM_INSERT);
        e->accept();
        return;

      case Key_Down:
        this->Cursor->cursor_off();   // Switch off cursor
        Deselect(this);
        dn_key();
        ValidateCursor(this);
        e->accept();
        return;

      case Key_Up:
        this->Cursor->cursor_off();   // Switch off cursor
        Deselect(this);
        up_key();
        ValidateCursor(this);
        e->accept();
        return;

      case Key_Left:
        this->Cursor->cursor_off();   // Switch off cursor
        Deselect(this);
        l_key();
        ValidateCursor(this);
        e->accept();
        return;

      case Key_Right:
        this->Cursor->cursor_off();   // Switch off cursor
        Deselect(this);
        r_key();
        ValidateCursor(this);
        e->accept();
        return;

      case Key_Home:
        this->Cursor->cursor_off();   // Switch off cursor
        Deselect(this);
        hm_key();
        ValidateCursor(this);
        e->accept();
        return;

      case Key_End:
        this->Cursor->cursor_off();   // Switch off cursor
        Deselect(this);
        end_key();
        ValidateCursor(this);
        e->accept();
        return;

      case Key_Next:
        this->Cursor->cursor_off();   // Switch off cursor
        Deselect(this);
        pdn_key();
        CentrateCursor(this);
        e->accept();
        return;

      case Key_Prior:
        this->Cursor->cursor_off();   // Switch off cursor
        Deselect(this);
        pup_key();
        CentrateCursor(this);
        e->accept();
        return;
    }
  }

  // Special keys with SHIFT modifier
  if (e->state() == ShiftButton) {
    switch (e->key()) {

      case Key_Left:
        this->Cursor->cursor_off();   // Switch off cursor
        Deselect(this);
        cl_key();
        ValidateCursor(this);
        e->accept();
        return;

      case Key_Right:
        this->Cursor->cursor_off();   // Switch off cursor
        Deselect(this);
        cr_key();
        ValidateCursor(this);
        e->accept();
        return;

      case Key_Home:
        this->Cursor->cursor_off();   // Switch off cursor
        Deselect(this);
        chm_key();
        ValidateCursor(this);
        e->accept();
        return;

      case Key_End:
        this->Cursor->cursor_off();   // Switch off cursor
        Deselect(this);
        cen_key();
        ValidateCursor(this);
        e->accept();
        return;
      }
    }

  // Special keys with CTRL modifier
  if (e->state() == ControlButton) {
    switch (e->key()) {

      case Key_C:                         // Copy accelerator
        if (check_copy())
          editActivated(IDM_COPY);
        e->accept();
        return;

      case Key_F:                         // Find accelerator
        if (check_find())
          optionsActivated(IDM_FIND);
        e->accept();
        return;

      case Key_G:                         // Goto accelerator
        if (check_goto())
          optionsActivated(IDM_GOTO);
        e->accept();
        return;

      case Key_N:                         // New accelerator
        fileActivated(IDM_NEW);
        e->accept();
        return;

      case Key_O:                         // Open accelerator
        fileActivated(IDM_OPEN);
        e->accept();
        return;

      case Key_P:
	if (check_spell())
	  optionsActivated(IDM_SPELL);    // Spell-check accelerator
        e->accept();
        return;

      case Key_Q:                         // Quit accelerator
        fileActivated(IDM_QUIT);
        e->accept();
        return;

      case Key_R:                         // Replace accelerator
        if (check_replace())
          optionsActivated(IDM_REPLACE);
        e->accept();
        return;

      case Key_S:                         // Save accelerator
        fileActivated(IDM_SAVE);
        e->accept();
        return;

      case Key_V:                         // Paste accelerator
        if (check_paste())
          editActivated(IDM_PASTE);
        e->accept();
        return;

      case Key_X:                         // Cut accelerator
        if (check_cut())
          editActivated(IDM_CUT);
        e->accept();
        return;
    }

  }

  // Special keys with ALT modifier
  if (e->state() == AltButton) {
    switch (e->key()) {

      case Key_P:                          // Reformat accelerator
        if (check_reformat())
          editActivated(IDM_REFORMAT);
        e->accept();
        return;
    }

  }

  // Normal characters (ascii code > 31 without CTRL and ALT)
  if (!(e->state() & (ControlButton | AltButton)) && e->ascii() > 31) {
   this->Cursor->cursor_off();      // Switch off cursor
   if (fmark) {
      temp[0] = e->ascii();
      temp[1] = (char)NULL;
      DelMark(this,temp);           // Subsitute highlighted area
      r_key();                      // Advance cursor
      ValidateCursor(this);
      }
    else
      insert(e->ascii(),fInsert,this);
    e->accept();
    return;
    }
  e->ignore();
}
