/***************************************************************************
 *                                                                         *
 *    File:         TeDLoad.cpp                                            *
 *                                                                         *
 *    Descrizione:  Caricamento file del sistema TeD X-Windows.            *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1995.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Data:         8 Ott 1995                                             *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "TeDFrame.h"
#include "TeD.h"

/*
** Funzione di caricamento del file. <pszFullFile> punta al nome
** del file da caricare. Ritorna TRUE in caso di successo,
** FALSE altrimenti.
*/

char FileOpen(char *pszFullFile)
{
  extern struct row *first;               /* puntatore alla prima riga */
  extern struct row *last;                /* puntatore all'ultima riga */
  extern struct row *actual;              /* puntatore alla riga cursore */
  extern struct row *video;               /* puntatore alla prima riga video */
  extern unsigned int first_video_row;    /* numero prima riga visualizzata */
  extern unsigned int first_video_line;   /* numero prima linea visualizzata */
  extern unsigned int last_video_row;     /* numero ultima riga visualizzata */
  extern unsigned int cursor_row;         /* numero riga del cursore */
  extern unsigned int cursor_line;        /* numero linea del cursore */
  extern unsigned int cursor_column;      /* numero colonna cursore */
  extern unsigned int t_len;              /* numero di righe del testo */
  extern unsigned int t_line;             /* numero di linee del testo */
  extern unsigned int n_mod;              /* flag di segnalazione modifica file */

  FILE *fload;                            /* puntatore al file */

  char input[ROWLEN + 1];                 /* riga in composizione */
  char read_char;                         /* lettura carattere dal file */
  char last_char = EOL;                   /* ultimo carattere letto */
  register int i;                         /* posizione prossimo carattere */
  register int j;                         /* posizione ultimo spazio */
  int first_line = 1;                     /* segnalazione scrittura prima riga */
  struct row *info;                       /* puntatore temporaneo */

  /* Caricamento del file */

  fload = fopen(pszFullFile,"rb");          /* apertura file */

  if (fload)                 /* apertura file riuscita */
    {     
    t_len = 0;               /* inizializzazione numero di righe del testo */
    t_line = 1;              /* inizializzazione numero di linee del testo */
    actual = first;          /* inizializzazione puntatore riga corrente */
    last = first;            /* inizializzazione puntatore ultima riga */
    *(first->text) = (char) NULL;   /* inizializzazione prima riga */

    i = 0;                   /* posizione prossimo carattere in ingresso */
    j = -1;                  /* inizializzazione posizione ultimo spazio */

    /* Caricamento del contenuto del file */

    while (read_char = getc(fload),feof(fload) == 0 && actual)
      {
      if (read_char == EOL)    /* hard return */
        {
          if (last_char == 32) {
             last_char = EOL;
          } else {
          next_line(&first_line);        /* allocazione nuova riga */
          if (actual)                    /* allocazione riga riuscita */
            {
            last = actual;
            *(input + i) = (char) NULL;
            strcpy(actual->text,input);
            actual->carriage = hard;
            }
            i = 0;
            j = -1;
          }
        }
      else if ((unsigned int)read_char > 31)
        {
        /* carattere normale */

        last_char = read_char;
        if (i < ROWLEN)                  /* riga non completa */
          {
          *(input + i) = read_char;
          if (read_char == 32) j = i;
          ++i;
          }

        else                             /* riga completa */
          {
           *(input + i) = read_char;
           if (read_char == 32) j = i;
           next_line(&first_line);
           if (actual)
             {
             /* allocazione riuscita */

             last = actual;
             if (j == -1)                  /* continuation mark */
               {
               strncpy(actual->text,input,ROWLEN);
               *(actual->text + ROWLEN) = (char) NULL;
               actual->carriage = cont;
               --t_line;
               *input = read_char;
               i = 1;
               }

             else                          /* soft return */
               {
                strncpy(actual->text,input,j);
                *(actual->text + j) = (char) NULL;
                actual->carriage = soft;
                i = ROWLEN - j;

                /* aggiorna buffer di ingresso */

                memmove(input,input + j + 1, (unsigned int) i);
                j = -1;
                }
              }
            }
          }
        }

    /* Chiusura file di ingresso */

    fclose(fload);

    /* scrittura eventuale ultima riga non terminata da hard return */

    if (actual && i)
      {
      next_line(&first_line);
      if (actual)
        {
        last = actual;
        strncpy(actual->text,input,i);            /* copia caratteri rimasti */
        *(actual->text + i) = (char) NULL;        /* termina la riga */
         }
      }

    /* Aggiornamento puntatori file */

    last->carriage = hard;                /* chiusura testo con hard return */

    while (actual->next)                  /* libera memoria in eccesso */
      {
      info = actual->next;
      actual->next = info->next;
      if (info->next)
         info->next->prev = actual;
      del_line(info);
      }

      actual = first;                          /* puntatore all'inizio */
      video = first;                           /* puntatore inizio visualizzazione */
      first_video_row = 0;                     /* prima riga visualizzata */
      first_video_line = 1;                    /* prima linea visualizzata */
      cursor_row = 0;                          /* riga cursore */
      cursor_line = 1;                         /* linea cursore */
      cursor_column = 0;                       /* colonna cursore */
      n_mod = 0;                               /* file non modificato */
     return(TRUE);                             /* caricamento positivo */
    }
  return(FALSE);                               /* caricamento negativo */
}

/*
** Programma per estrarre il pathname dalla stringa in e scriverlo nella stringa out.
*/

void ParseName(char *in,char *out)
{
  int i;

  for (i = strlen(in)-1; i >= 0; --i)
    if (*(in+i) == '/') break;

  strcpy(out,in);
  strcpy(out+i+1,"");
}


/*
** Funzione di aggiunta di una nuova riga. Se manca memoria il puntatore
** actual diviene NULL. Non aggiunge una nuova riga se <first_line> e'
** uguale a 1. <first_line> viene sempre posto a zero.
*/

void next_line(int *first_line)
{
  extern struct row *actual;
  extern unsigned int t_len;
  extern unsigned int t_line;

  if (*first_line == 1)           /* prima riga */
    {
      *first_line = 0;
      return;
    }

  if (actual->next)
    {
      actual = actual->next;      /* prossima riga */
      ++t_len;                    /* aggiorna numero righe */
      ++t_line;                   /* aggiorna numero linee */
    }
  else
    actual = add(actual);         /* aggiunta nuova riga */

  *first_line = 0;
}