/***************************************************************************
 *                                                                         *
 *    File:         TeDMenu.cpp                                            *
 *                                                                         *
 *    Descrizione:  Impostazione menu del sistema TeD X-Windows            *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Data:         29 Ago 1994                                            *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <qwidget.h>                 // Widgets
#include <qmenubar.h>                // Menu Bar
#include <qmenudta.h>                // Menu Data
#include <qaccel.h>                  // Accelerators keys
#include <qkeycode.h>                // Key codes
#include <qfont.h>                   // Fonts
#include "TeDFrame.h"
#include "TeD.h"

/*
** Menu bar creator
*/

void MainWindow::MenuCreate(void)
{

    const char *familys[] = {"Arial", "Charter", "Clean", "Courier", "Fixed",
                             "Gothic", "Helvetica", "Lucida", "Lucidabright",
                             "Lucidatypewriter", "Mincho",
                             "New century Schoolbook",
                             "Symbol", "Terminal", "Times", "Utopia", 0};

    const char *pSizes[]  = {  "8", "10", "12", "14", "16", "18", "24", "36",
                                0 };
    const char **tmp;

    // Allocates menu bar and popup menus
    menu = new QMenuBar(this);
    FileMenu = new QPopupMenu;
    EditMenu = new QPopupMenu;
    OptionsMenu = new QPopupMenu;
    FontMenu = new QPopupMenu;
    FamilyMenu = new QPopupMenu;
    SizeMenu = new QPopupMenu;
    HelpMenu = new QPopupMenu;
    
    // Set menus font
    QFont MenuFont("lucidabright",14, QFont::Bold,TRUE);
    menu->setFont(MenuFont);
    FileMenu->setFont(MenuFont);
    EditMenu->setFont(MenuFont);
    OptionsMenu->setFont(MenuFont);
    FontMenu->setFont(MenuFont);
    FamilyMenu->setFont(MenuFont);
    SizeMenu->setFont(MenuFont);
    HelpMenu->setFont(MenuFont);

    // Load menu bar items
    menu->move(0,0);
    menu->insertItem( "File", FileMenu, IDM_FILE);
    menu->insertItem( "Edit", EditMenu, IDM_EDIT);
    menu->insertItem( "Options", OptionsMenu, IDM_OPTIONS);
    menu->insertItem( "Font", FontMenu, IDM_FONT);
    menu->insertItem( "Help", HelpMenu, IDM_HELP);

    // Load file menu items
    FileMenu->insertItem("New\tCtrl-N", IDM_NEW);
    FileMenu->insertItem("Open...\tCtrl-O", IDM_OPEN);
    FileMenu->insertSeparator(-1);
    FileMenu->insertItem("Save\tCtrl-S", IDM_SAVE);
    FileMenu->insertItem("Save As...", IDM_SAVEAS);
    FileMenu->insertItem("Autosave...", IDM_AUTOSAVE);
    FileMenu->insertSeparator(-1);
    FileMenu->insertItem("Quit\tCtrl-Q", IDM_QUIT);

    // Load edit menu items
    EditMenu->insertItem("Cut\tCtrl-X", IDM_CUT);
    EditMenu->insertItem("Copy\tCtrl-C", IDM_COPY);
    EditMenu->insertItem("Paste\tCtrl-V", IDM_PASTE);
    EditMenu->insertSeparator(-1);
    EditMenu->insertItem("Clear\tDel", IDM_CLEAR);
    EditMenu->insertSeparator(-1);
    EditMenu->insertItem("Reformat As One Paragraph\tAlt-P", IDM_REFORMAT);

    // Load options menu items
    OptionsMenu->insertItem("Find...\tCtrl-F", IDM_FIND);
    OptionsMenu->insertItem("Replace...\tCtrl-R", IDM_REPLACE);
    OptionsMenu->insertItem("Spell check...\tCtrl-P", IDM_SPELL);
    OptionsMenu->insertSeparator(-1);
    OptionsMenu->insertItem("Goto line...\tCtrl-G", IDM_GOTO);
    OptionsMenu->insertSeparator(-1);
    OptionsMenu->insertItem("Overstrike\tIns", IDM_INSERT);

    // Load font menu items
    FontMenu->insertItem("Family", FamilyMenu, IDM_FAMILY);
    FontMenu->insertItem("Size", SizeMenu, IDM_SIZE);

    // Load family menu items
    tmp = familys;
    while( *tmp )
        FamilyMenu->insertItem( *tmp++ );

    // Load size menu items
    tmp = pSizes;
    while( *tmp )
        SizeMenu->insertItem( *tmp++ );

    // Load help menu items
    HelpMenu->insertItem("Product Information...", IDM_PRODUCTINFO);

    // Connect Items to function managers
    connect(menu,SIGNAL(highlighted(int)),SLOT(menuActivated(int)));
    connect(FileMenu,SIGNAL(activated(int)),SLOT(fileActivated(int)));
    connect(EditMenu,SIGNAL(activated(int)),SLOT(editActivated(int)));
    connect(OptionsMenu,SIGNAL(activated(int)),SLOT(optionsActivated(int)));
    connect(HelpMenu,SIGNAL(activated(int)),SLOT(helpActivated(int)));
    connect(FamilyMenu, SIGNAL(activated(int)), SLOT(familyActivated(int)) );
    connect(SizeMenu, SIGNAL(activated(int)), SLOT(pSizeActivated(int)) );
}

// Menu bar manager

void MainWindow::menuActivated(int idx)
{

  extern char fFind;
  extern char fReplace;
  extern char fGoto;
  extern unsigned int t_len;
  extern struct row *first;


  switch (idx) {
    case IDM_FILE:

      // Check text modified
      FileMenu->setItemEnabled(IDM_SAVE, check_save());

      // Check autosave
      FileMenu->setItemEnabled(IDM_AUTOSAVE, check_autosave());
      break;

    case IDM_EDIT:

      // Check cut
      EditMenu->setItemEnabled(IDM_CUT, check_cut());

      // Check copy
      EditMenu->setItemEnabled(IDM_COPY, check_copy());

      // Check paste
      EditMenu->setItemEnabled(IDM_PASTE, check_paste());

      // Check clear
      EditMenu->setItemEnabled(IDM_CLEAR, check_clear());

      // Check reformat paragraph
      EditMenu->setItemEnabled(IDM_REFORMAT, check_reformat());

      break;
     

    case IDM_OPTIONS:

       // Check find
       OptionsMenu->setItemEnabled(IDM_FIND, check_find());

       // Check replace
       OptionsMenu->setItemEnabled(IDM_REPLACE, check_replace());

       // Check spell checking
       OptionsMenu->setItemEnabled(IDM_SPELL, check_spell());

       // Check goto
       OptionsMenu->setItemEnabled(IDM_GOTO, check_goto());
       break;
     }
}

// File menu manager

void MainWindow::fileActivated(int idx)
{
  switch (idx) {

    case IDM_NEW:
      NewFile();
      break;

    case IDM_OPEN:
      OpenFile();
      break;

    case IDM_SAVE:
      {
      extern char fUntitled;
      if (fUntitled)                  // File without name
        SaveAsFile();
      else                            // File with name
        MenuSave(this);
      }
      break;

    case IDM_SAVEAS:
      SaveAsFile();
      break;

    case IDM_AUTOSAVE:
      {
      extern char fUntitled;
      if (!fUntitled)
        asave = new WinAutosave(this);
      }
      break;

    case IDM_QUIT:
      close (FALSE);
      break;

  }

}

// Edit menu manager

void MainWindow::editActivated(int idx)
{
  switch (idx) {

    case IDM_COPY:
      if (check_copy()) {
        if (!CopyMark(this))
          MessageBox("Clipboard: Error copying text", MB_OK);
      }
      break;

    case IDM_CUT:
      this->Cursor->cursor_off();   // Switch off cursor
      if (check_cut()) {
        if (!CopyMark(this))
          MessageBox("Clipboard: Error copying text", MB_OK);
        else 
          DelMark(this,"");
        }
      break;

    case IDM_PASTE:
      if (check_paste()) {
        if (!PasteClipboard(this))
            MessageBox("Clipboard: Error pasting text", MB_OK);
       }
      break;

    case IDM_CLEAR:
      {
      extern char fmark;
      this->Cursor->cursor_off();   // Switch off cursor
      if (fmark)
        DelMark(this,"");
      else
        del_key(this);
      }
      break;

    case IDM_REFORMAT:
      this->Cursor->cursor_off();   // Switch off cursor
      Reformat(this);
      break;
    }

}


// Options menu manager

void MainWindow::optionsActivated(int idx)
{
   switch(idx) {

   case IDM_INSERT:                  /* Modifica tasto inserimento */
     extern char fInsert;
     fInsert = !fInsert;
     if (fInsert)
       OptionsMenu->changeItem("Overstrike\tIns",IDM_INSERT);
     else
       OptionsMenu->changeItem("Insert\tIns",IDM_INSERT);

     // Update bottom line
     UpdateBottomLine();
     break;

   case IDM_GOTO:
     this->Cursor->cursor_off();
     gotoline = new WinGoto (this);
     break;

   case IDM_FIND:
     find = new WinFind (this);
     break;

   case IDM_REPLACE:
     subs = new WinSubs (this);
     break;

   case IDM_SPELL:
     spell = new WinSpell (this);
     break;
   }
}


// Family menu manager
void MainWindow::familyActivated( int id )
{
  extern char fontname[];                      // Global font name
  extern int FontHeight;                       // Font height

  // Switch off cursor
  this->Cursor->cursor_off();

  // Copy name of the font to be used
  strcpy(fontname, ((QPopupMenu*)sender())->text(id));

  // Change font metrics informations
  SetFontMetrics();

  // Reallocate cursor pixmap
  delete this->Cursor->pm;
  this->Cursor->pm = new QPixmap (2, FontHeight);

  // Repaint text widget
  this->Text->repaint();

  // Set cursor
  SetCursor(this);
}


// Size menu manager
void MainWindow::pSizeActivated( int id )
{
  extern int size;                              // Global font size
  extern int FontHeight;                        // Font height

  // Switch off cursor
  this->Cursor->cursor_off();

  // Copy size of the font to be used
  size = atoi( ( (QPopupMenu*)sender())->text( id ) );

  // Change font metrics informations
  SetFontMetrics();

  // Reallocate cursor pixmap
  delete this->Cursor->pm;
  this->Cursor->pm = new QPixmap (2, FontHeight);

  // Repaint text widget
  this->Text->repaint();

  // Set cursor
  SetCursor(this);
}


// Help menu manager

void MainWindow::helpActivated(int idx)
{
  product = new WinProduct;
}
