/***************************************************************************
 *                                                                         *
 *    File:         TeDMessage.cpp                                         *
 *                                                                         *
 *    Descrizione:  Message box del sistema TeD X-Windows                  *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Data:         8 Ott 1994                                             *
 *                                                                         *
 ***************************************************************************/

#include <qdialog.h>                       // Dialog box
#include <qpushbt.h>                       // Push buttons
#include <qlabel.h>                        // Labels
#include <qframe.h>                        // Frames
#include "TeDFrame.h"
#include "TeD.h"

// Message box handler

int MainWindow::MessageBox(char* text, int type)
{

  message = new WinMessageBox(text, type, this, "Message", TRUE, 0);

  return(answer);  
}


// Message box window constructor

WinMessageBox::WinMessageBox(char* text, int type, QWidget* parent,
                             const char* name, bool modal, WFlags f)
              : QDialog (0, name, modal, f)
{

  // Resets type
  Mtype = type;
  
  // Sets frame
  frame = new QFrame(this);
  separator = new QFrame(this);
  frame->setFrameStyle( QFrame::Panel | QFrame::Raised );
  separator->setFrameStyle( QFrame::HLine | QFrame::Sunken );

  // Sets label
  Mlabel = new QLabel(this);
  Mlabel->setText(text);
  Mlabel->setAlignment(AlignCenter);
  Mlabel->setAutoResize(TRUE);

  // Checks type of message box

  switch(type) {
    case MB_OK:

      // Allocates ok button and connects to slot
      Yes = new QPushButton(this);
      Yes->setText("Ok");
      connect(Yes,SIGNAL(clicked()),SLOT(yes_clicked()));      
      break;

    case MB_YESNOCANCEL:
      // Allocates yes no cancel buttons and connects to slot
      Yes = new QPushButton(this);
      No = new QPushButton(this);
      Cancel = new QPushButton(this);

      Yes->setText("Yes");
      No->setText("No");
      Cancel->setText("Cancel");

      connect(Yes,SIGNAL(clicked()),SLOT(yes_clicked()));
      connect(No,SIGNAL(clicked()),SLOT(no_clicked()));
      connect(Cancel,SIGNAL(clicked()),SLOT(cancel_clicked()));     
      break;
    }

  // Connects answer
  connect (this, SIGNAL(answer(int)), parent, SLOT(MessageAnswer(int)));

  // Resizes window and shows it
  resize(300,120);
  show();
}

// Yes button clicked

void WinMessageBox::yes_clicked()
{
  emit answer(MBID_YES);
  close(FALSE);
}

// No button clicked

void WinMessageBox::no_clicked()
{
  emit answer(MBID_NO);
  close(FALSE);
}

// Cancel button clicked

void WinMessageBox::cancel_clicked()
{
  emit answer(MBID_CANCEL);
  close(FALSE);
}


// Resizing event of the message box

void WinMessageBox::resizeEvent( QResizeEvent* )
{

  const int space = 20;
  const int bspace = 30;
  const int hei = 25;
  int blen = (this->width() - 4*space)/3;

  // Sets frame geometry
  frame->setGeometry(0, 0, this->width(), this->height());
  separator->setGeometry(0, bspace + hei , this->width(), 4);

  // Sets label geometry
  Mlabel->setGeometry(space, space, this->width() - space - space , hei);

  // Sets buttons position
  switch(Mtype) {
    case MB_OK:
      Yes->setGeometry(blen+2*space, this->height() - space - hei, blen, hei);
      break;

    case MB_YESNOCANCEL:
      Yes->setGeometry(space, this->height() - space - hei, blen, hei);
      No->setGeometry(blen+2*space, this->height() - space - hei, blen, hei);
      Cancel->setGeometry(2*blen+3*space, this->height() - space - hei,
                          blen, hei);
      break;
  }
}


// Message box destructor

WinMessageBox::~WinMessageBox()
{
  // Free all storage
   delete Mlabel;                    // Message box label
   delete Yes;                       // Yes button
   delete separator;                 // Separator line
   delete frame;                     // Frame

   if (Mtype == MB_YESNOCANCEL) {
     delete No;                 // No button
     delete Cancel;             // Cancel button
     }
}

// Slot receiving answer of the message box

void MainWindow::MessageAnswer(int a)
{

   // Copy answer
   answer = a;

}
