/***************************************************************************
 *                                                                         *
 *    File:         TeDMouse.cpp                                           *
 *                                                                         *
 *    Description:  Mouse manager of X-Window TeD                          *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1995.                     *
 *                  All rights reserved.                                   *
 *                                                                         *
 *    Date:         28 Dec 1995                                            *
 *                                                                         *
 ***************************************************************************/

#include <qwidget.h>                       // Widget class
#include <qevent.h>                        // Event class
#include <qrect.h>                         // Rectangles
#include "TeDFrame.h"
#include "TeD.h"


// Mouse move event of the main window

void MainWindow::mouseMoveEvent (QMouseEvent* e)
{
    QPoint p;
    extern QRect rcClient;   // Text window
    p = e->pos();            // Mouse position

    if (p.x() < rcClient.right() && p.y() > menu->height()
        && p.y() - menu->height() < rcClient.bottom() ) {
       // Sets cursor shape
       setCursor(ibeamCursor);
       }
    else {
       setCursor(arrowCursor);
     }

}

// Mouse click event of the text window

void TextWidget::mousePressEvent (QMouseEvent* e)
{
  char fOutside;                         // Click outside area
  int x, y;                              // Coordinates of click
  MainWindow *hwnd;                      // Pointer to main window
  extern int FontMaxWidth;               // Font max width
  extern int FontDescent;                // Font descent
  extern QRect rcClient;                 // Painting area

  // Gets handler of main window
  hwnd = (MainWindow *)parentWidget();

  // Check which button is pressed

  switch (e->button()) {

    case (LeftButton):
      x = e->pos().x();
      y = e->pos().y();

      // Determine whether click is outside window
      fOutside = ((x < rcClient.left() + FontMaxWidth)   ||
                  (y > rcClient.bottom() - FontDescent)  ||
                  (x > rcClient.right() - FontMaxWidth)  ||
                  (y < rcClient.top() + FontDescent));

       if (e->state() & ShiftButton ) {   // shift is down: extend selection
           TextSelect(hwnd,TRUE,x,y,fOutside); 
           return;
           }
       else {

          // switch off cursor
          hwnd->Cursor->cursor_off();

          // clear selection
          TextSelect(hwnd,FALSE,x,y,fOutside);
          CursorAt(x,y);                          // move cursor
          ValidateCursor(hwnd);                   // validate cursor
          return;
          }

     case (RightButton):                          // Copy selected text
       if (check_copy()) {
         if (!CopyMark(hwnd))
           hwnd->MessageBox("Clipboard: Error copying text", MB_OK);
         }
       return;

     case (MidButton):                            // Paste text
      if (check_paste()) {
        if (!PasteClipboard(hwnd))
            hwnd->MessageBox("Clipboard: Error pasting text", MB_OK);
       }
       return;
     }
}

// Mouse double click event of the text window

void TextWidget::mouseDoubleClickEvent (QMouseEvent* e)
{
  MainWindow *hwnd;

  if (e->button() ==  LeftButton) {

      // Gets main window handler
      hwnd = (MainWindow *)parentWidget();

      // Switch off cursor
      hwnd->Cursor->cursor_off();

      WordSelect(hwnd, e->pos().x(), e->pos().y());
      }
}


// Mouse move event of the text window: extend selection

void TextWidget::mouseMoveEvent (QMouseEvent* e)
{
  char fOutside;                         // Move outside area
  MainWindow *hwnd;                      // Pointer to main window
  extern int FontMaxWidth;               // Font max width
  extern int FontDescent;                // Font descent
  extern QRect rcClient;                 // Painting area

  if (e->state() ==  LeftButton) {

    // Save mouse position
    mousex = e->pos().x();
    mousey = e->pos().y();

    // Determine whether move is outside window
    fOutside = ((mousex < rcClient.left() + FontMaxWidth)   ||
                (mousey > rcClient.bottom() - FontDescent)  ||
                (mousex > rcClient.right() - FontMaxWidth)  ||
                (mousey < rcClient.top() + FontDescent));


    // Extend selection if timer is not active and activate timer if necessary
    if (!timer->isActive()) {

      // Gets handler of main window
      hwnd = (MainWindow *)parentWidget();

      TextSelect(hwnd, TRUE, mousex, mousey, fOutside);
      if (fOutside)
        timer->start(300);
      }
    else if (!fOutside)
      timer->stop();           // Stops timer
    }
}


// Mouse release event of the text widget: stop timer

void TextWidget::mouseReleaseEvent (QMouseEvent *e)
{

  // Check whether left button is pressed and timer is running
  if (e->button() ==  LeftButton && timer->isActive())
    timer->stop();

}



// Scroll selection

void TextWidget::ScrollSelection()
{

  TextSelect((MainWindow *)parentWidget(), TRUE, mousex, mousey, TRUE);

} 