/***************************************************************************
 *                                                                         *
 *    File:         TeDPaste.cpp                                           *
 *                                                                         *
 *    Description:  Clipboard copy of X-Window TeD                         *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Date:         29 Aug 1994                                            *
 *                                                                         *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "TeDFrame.h"
#include "TeD.h"

/*
** Funzione per leggere una riga da una stringa formattata. Ritorna il tipo di
** ritorno carrello. <in> punta alla stringa in ingresso. La riga viene scritta
** sulla stringa cui punta <out>, che deve avere una lunghezza di almeno
** ROWLEN + 2 caratteri. <pos> punta ad una variabile unsigned int che
** contiene la posizione da cui bisogna iniziare a leggere la stringa.
** <len> deve contenere la lunghezza della stringa cui punta <in>.
** La funzione aggiorna il valore puntato da <pos> per la chiamata successiva.
** N.B. Quando *pos = 0 si assume che sia la prima chiamata alla funzione: di
** conseguenza si resetta il valore dell'ultimo carattere letto. Se *pos > 0
** si assume che si tratti delle chiamate successive e si utilizza come
** ultimo carattere letto l'ultimo carattere letto nella chiamata precedente.
*/

enum ret ReadFormattedString(char *in, char *out, unsigned int *pos,
                             unsigned int len)
{
  char read_char;                         /* lettura carattere dalla stringa */
  static char last_char;                  /* ultimo carattere letto */
  register int i;                         /* posizione prossimo carattere */
  register int j;                         /* posizione ultimo spazio */

  /* inizializza variabili */

  i = 0;
  j = -1;
  if (*pos == 0)
     last_char = EOL;

  while (*pos <= len) {
    if ((read_char = *(in + (*pos)++)) == EOL) {  /* hard return */
      if (last_char == 32) {               /* ignora hard return preceduto */
        last_char = EOL;                   /* da uno spazio */
        }
      else {
        *(out + i) = (char) NULL;          /* termina stringa */
        return(hard);                      /* ritorna hard return */
        }
      }
    else if ((unsigned int)read_char > 31) {        /* carattere normale */
      last_char = read_char;          /* salva ultimo carattere letto */
      *(out + i) = read_char;         /* scrive carattere */
      if (read_char == 32) j = i;     /* posizione ultimo spazio */
      if (i < ROWLEN)                 /* riga non completa */
        ++i;
      else {                             /* riga completa */
        if (j == -1) {                   /* continuation mark */
          *(out + ROWLEN) = (char) NULL; /* termina stringa */
          --*pos;                  /* arretra al primo carattere da leggere */
          return(cont);            /* ritorna continuation mark */
          }
        else {                          /* soft return */
          *(out + j + 1) = (char) NULL; /* termina riga in composizione */
          *pos -= ROWLEN - j;   /* arretra primo carattere da leggere */
          return(soft);         /* ritorna soft mark */
          }
        }
      }
    }

  /* scrittura eventuale ultima riga non terminata da hard return */

  *(out + i) = (char) NULL;        /* termina la riga */
  return(cont);                    /* ritorna continuation mark */
}



/*
** Funzione per copiare il testo della clipboard alla posizione
** attuale del cursore. Se esiste del testo marcato, provvede
** a cancellarlo. Ritorna TRUE in caso di successo, FALSE in caso
** di errore di apertura della clipboard. <hwnd> punta alla finestra
** client.
*/

char PasteClipboard(MainWindow *hwnd)
{
  extern char fmark;                  // Segnalazione presenza testo marcato
  extern unsigned int cursor_column;  // Colonna cursore
  extern unsigned int cursor_row;     // Riga cursore
  char *ins_line;                     // Puntatore stringa clipboard

  ins_line = StringFromClipboard(hwnd);  // Legge clipboard
  if (!ins_line)
    return(FALSE);                    // Errore apertura clipboard

  hwnd->Cursor->cursor_off();         // Switch off cursor

  if (fmark)
    DelMark(hwnd,ins_line);           // Sostituisce testo marcato

  else {
    // Inserisce testo clipboard e aggiorna posizione cursore
    mov_cur(justfy(hwnd,cursor_column,cursor_row,ins_line));
    ValidateCursor(hwnd);
    modify(hwnd);
    }

  // Free allocated space and exit
  delete[] ins_line;
  return(TRUE);
}
