/***************************************************************************
 *                                                                         *
 *    File:         TeDProduct.cpp                                         *
 *                                                                         *
 *    Description:  Product info of X-Window TeD                           *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1996.                     *
 *                  All rights reserved.                                   *
 *                                                                         *
 *    Date:         14 Jan 1996                                            *
 *                                                                         *
 ***************************************************************************/

#include <qlabel.h>                      // Labels
#include <qdialog.h>                     // Dialogs
#include <qframe.h>                      // Frames
#include <qfont.h>                       // Fonts
#include <qpushbt.h>                     // Push buttons
#include "TeDFrame.h"
#include "TeD.h"

// Product info window constructor

WinProduct::WinProduct(QWidget* parent, const char* name, WFlags f)
        : QDialog (0, name, TRUE, f)
{

  // Sets frame & separator
  frame = new QFrame(this);
  separator = new QFrame(this);
  frame->setFrameStyle( QFrame::Panel | QFrame::Raised );
  separator->setFrameStyle( QFrame::HLine | QFrame::Sunken );

  // Defines font
  QFont font("Charter", 48, QFont::Bold);

  // Set labels
  Mlabel1 = new QLabel(this);
  Mlabel1->setText("TeD");
  Mlabel1->setAlignment(AlignCenter);
  Mlabel1->setAutoResize(TRUE);
  Mlabel1->setFont(font);

  Mlabel2 = new QLabel(this);
  Mlabel2->setText("Version 1.1");
  Mlabel2->setAlignment(AlignCenter);
  Mlabel2->setAutoResize(TRUE);
  font.setPointSize(18);
  Mlabel2->setFont(font);

  Mlabel3 = new QLabel(this);
  Mlabel3->setText("Copyright 1996 by");
  Mlabel3->setAlignment(AlignCenter);
  Mlabel3->setAutoResize(TRUE);
  Mlabel3->setFont(font);

  Mlabel4 = new QLabel(this);
  Mlabel4->setText("Paolo Galatola");
  Mlabel4->setAlignment(AlignCenter);
  Mlabel4->setAutoResize(TRUE);
  Mlabel4->setFont(font);

  // Sets ok button
  Ok = new QPushButton(this);
  Ok->setText("Ok");

  // Connect ok button signal
  connect(Ok, SIGNAL(clicked()), SLOT(ok_clicked()));

  // Resize window & show
  resize(270, 300);
  show();
}

// Product info window destructor

WinProduct::~WinProduct()
{
  // Free storage

  delete Mlabel1;                // Message box label
  delete Mlabel2;                // Message box label
  delete Mlabel3;                // Message box label
  delete Mlabel4;                // Message box label
  delete Ok;                     // Ok button
  delete separator;              // Separator line
  delete frame;                  // Frame
}



// Product info window resize event

void WinProduct::resizeEvent( QResizeEvent* )
{
  const int space = 20;
  const int sspace = 10;
  const int bspace = 30;
  const int hei = 25;
  const int lhei1 = 50;
  const int lhei2 = 30;
  int blen = (this->width() - 3 * space) / 2;

  // Sets frame geometry
  frame->setGeometry(0, 0, this->width(), this->height());

  // Sets separator geometry
  separator->setGeometry(0, height() - space - hei, this->width(), 4);  

  // Sets labels geometry
  Mlabel1->setGeometry(sspace, bspace, width() - space, lhei1);
  Mlabel2->setGeometry(sspace, bspace + bspace + lhei1, width() - space,
                       lhei2);
  Mlabel3->setGeometry(sspace, bspace + bspace + lhei1 + lhei2,
                       width() - space, lhei2);
  Mlabel4->setGeometry(sspace, bspace + bspace + lhei1 + 2 * lhei2,
                       width() - space, lhei2);

  // Set ok button geometry
  Ok->setGeometry ((width() + 3 * space) / 4, this->height() - sspace - hei,
                   blen, hei);
}


// Ok button pressed

void WinProduct::ok_clicked()
{

  // Close window
  close (FALSE);
}
