/***************************************************************************
 *                                                                         *
 *    File:         TeDSave.cpp                                            *
 *                                                                         *
 *    Description:  File saving of X-Window TeD                            *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Date:         29 Aug 1994                                            *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <qstring.h>                        // Qt Strings
#include <qfiledlg.h>                       // File dialog
#include "TeDFrame.h"
#include "TeD.h"


/*
** Funzione di salvataggio del file. <name> contiene il nome del file da
** scrivere. Ritorna TRUE in caso di successo, FALSE altrimenti.
*/

char SaveFile(char *name)
{
  extern struct row *first;      // pointer to first row
  struct row *now;               // pointer
  char out[ROWLEN + 4];          // row under printing
  register int i;                // counter
  FILE *file_write;              // file pointer

  // Open file to write

  if (!(file_write = fopen(name,"wb")))
    return(FALSE);

  // Initialize pointer

  now = first;

  while (now) {
    strcpy(out,now->text);
    WriteString(out,now->carriage);
    for (i = 0; i < strlen(out); ++i) {
       if (fputc(*(out + i),file_write) == EOF) {
         fclose(file_write);
         return(FALSE);
         }
       }
    now = now->next;
    }

  // Close file and return

  if(fclose(file_write))
    return(FALSE);
  return(TRUE);
}

/*
** Funzione di gestione delle operazioni di salvataggio. Aggiorna il
** contatore delle modifiche e chiama la funzione di scrittura del file.
** Avverte con una message box in caso di errore nella scrittura del file.
** <hwnd> punta alla finestra client.
*/

void MenuSave(MainWindow *hwnd)
{
  extern unsigned int n_mod;
  extern char szFullFile[];

   if (!SaveFile(szFullFile)) {      // error creating file

     hwnd->MessageBox("Save: Error writing file", MB_OK);
     return;
     }
   n_mod = 0;
}


/*
** Funzione di gestione delle operazioni di salvataggio con nome.
*/

void MainWindow::SaveAsFile(void)
{
  extern char szFullFile[];               // nome file precedente
  extern char fUntitled;                  // flag file senza titolo
  QString OpenFileName;
  char dirName[CCHMAXPATH];

  // Gets the directory name
  ParseName(szFullFile,dirName);

  // Pops-up save as window
  OpenFileName = QFileDialog::getSaveFileName (dirName, "*", this);

  if (OpenFileName.isNull())               // no selectioned name
       return;
  else {                                   // name selectioned   
 
     // Extracts string name
     strcpy(dirName,(OpenFileName));
     char Title[CCHMAXPATH+6] = "TeD - ";
     extern unsigned int n_mod;
     strcpy(szFullFile,dirName);           // copy name
     fUntitled = FALSE;                    // signal not untitled
     if (n_mod == 0)
       ++n_mod;                            // signal modify
     strcat(Title,szFullFile);

     // Set application name
     setCaption(Title);

     // call save file
     MenuSave(this);                       
     }
}
