/***************************************************************************
 *                                                                         *
 *    File:         TeDScroll.cpp                                          *
 *                                                                         *
 *    Description:  Scrollbar manager of X-Window TeD                      *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Date:         29 Aug 1994                                            *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <qpaintd.h>                      // Paint device
#include <qrect.h>                        // Rectangles
#include <qwidget.h>                      // Widgets
#include "TeDFrame.h"
#include "TeD.h"

/*
** Function to scroll screen. dev must point to the paint device. dd is
** the scrolling amount. dir is the direction: if zero it indicates
** vertical scrolling, otherwise horizontal scrolling. The video is
** automatically updated.
*/

void Scroll (QPaintDevice *dev, int dd, int dir)
{
  int dx, dy;               // Destination
  int sx, sy;               // Source
  int sw, sh;               // Width and height of source rectangle
  int x, y, w, h;           // Update rectangle
  extern QRect rcClient;    // Painting screen

  if (dir)                   // Horizontal scrolling
    {
    dy = rcClient.y();
    sy = dy;
    y = sy;
    sh = rcClient.height();
    h = sh;
    if (dd > 0)
      {
      sx = rcClient.x();
      dx = sx + dd;
      sw = rcClient.width() - dd;
      x = sx;
      w = dd;
      }
    else
      {
      dx = rcClient.x();
      sx = dx - dd;
      sw = rcClient.width() + dd;
      x = rcClient.right() + dd;
      w = -dd;
      }
    }
  else                       // Vertical scrolling
    {
    dx = rcClient.x();
    sx = dx;
    x = dx;
    w = rcClient.width();
    sw = w;
    if (dd > 0)
      {
      sy = rcClient.y();
      dy = sy + dd;
      y = rcClient.top();
      h = dd;
      sh = rcClient.height() - dd;
      }
    else
      {
      dy = rcClient.y();
      sy =  dy - dd;
      sh = rcClient.height() + dd;
      y = rcClient.bottom() + dd + 1;
      h = -dd;
      }
    }

  // Scroll screen
  bitBlt(dev, dx, dy, dev, sx, sy, sw, sh, CopyROP);

  // Repaint screen
  ((QWidget *)dev)->repaint(x,y,w,h);  
}


/*
** Function to move on a given row.
*/

void MainWindow::SetVPosition(int i)
{
  extern unsigned int first_video_row;
  extern unsigned int first_video_line;
  extern unsigned int t_len;
  extern struct row *video;
  extern int FontHeight;                  // Interline spacing
  extern QRect rcClient;

  int scroll;
  scroll = first_video_row;

  if (i > first_video_row) {

    this->Cursor->cursor_off();                    // Switch off cursor
    while (i > first_video_row && video->next) {
      ++first_video_row;
      if (video->carriage != cont) ++first_video_line;
      video = video->next;
    } /* endwhile */
   scroll = (scroll - first_video_row) * FontHeight;
   if (- scroll < rcClient.height())
      Scroll(this->Text,scroll,0);
   else
      this->Text->repaint();

   // Set cursor
   SetCursor(this);

  } else if (i < first_video_row) {

      this->Cursor->cursor_off();                    // Switch off cursor
      while (i < first_video_row && video->prev) {
        --first_video_row;
        video = video->prev;
        if (video->carriage != cont) --first_video_line;
      } /* endwhile */
    scroll = (scroll - first_video_row) * FontHeight;
    if (scroll < rcClient.height())
      Scroll(this->Text,scroll,0);
    else
       this->Text->repaint();

   // Set cursor
   SetCursor(this);

  }/* endif */
}

/*
** Function to signal vertical slider pressed.
*/

void MainWindow::VSliderPressed()
{
  extern char fVertTrack;               // Vertical tracking

  fVertTrack = TRUE;
}

/*
** Function to signal vertical slider released.
*/

void MainWindow::VSliderReleased()
{
  extern char fVertTrack;               // Vertical tracking
  extern unsigned int t_len;            // lenght of text  
  extern unsigned int first_video_row;  // number first video row
  extern unsigned int last_video_row;   // number first video row

  fVertTrack = FALSE;
  vscroll->setSteps (1, (int)last_video_row - (int)first_video_row + 1);
}


/*
** Function to update horizontal position
*/

void MainWindow::SetHPosition(int i)
{
  extern int LeftXScreen;
  int scroll;

  scroll = LeftXScreen - i;
  LeftXScreen = i;

  this->Cursor->cursor_off();                    // Switch off cursor  
  Scroll(this->Text,scroll,1);

  SetCursor(this);                               // Set cursor
}

/*
** Function to signal horizontal slider pressed.
*/

void MainWindow::HSliderPressed()
{
  extern char fHorzTrack;               // Horizontal tracking

  fHorzTrack = TRUE;
}

/*
** Function to signal horizontal slider released.
*/

void MainWindow::HSliderReleased()
{
  extern char fHorzTrack;               // Horizontal tracking
  extern int MaxXScreen;                // Maximum x coordinate text

  fHorzTrack = FALSE;
}

/*
** Funzione per retrocedere di una riga. L'handler punta alla finestra client.
*/

void LineUp(MainWindow *hwnd)
{
  extern unsigned int first_video_row;
  extern unsigned int first_video_line;
  extern struct row *video;
  extern int FontHeight;

  if (first_video_row > 0) {
    --first_video_row;
    video = video->prev;
    if (video->carriage != cont) --first_video_line;
    hwnd->Cursor->cursor_off();
    Scroll(hwnd->Text, FontHeight, 0);
    SetCursor(hwnd);
  }
}

/*
** Funzione per avanzare di una riga. L'handler punta alla finestra client.
*/

void LineDown(MainWindow *hwnd)
{
  extern unsigned int first_video_row;
  extern unsigned int first_video_line;
  extern unsigned int t_len;
  extern struct row *video;
  extern int FontHeight;

  if (first_video_row < t_len) {
    ++first_video_row;
    if (video->carriage != cont) ++first_video_line;
    video = video->next;
    hwnd->Cursor->cursor_off();
    Scroll(hwnd->Text, -FontHeight, 0);
    SetCursor(hwnd);
  }
}

/*
** Funzione per spostarsi a sinistra di un carattere.
** L'handler punta alla finestra client.
*/

void LineLeft(MainWindow *hwnd)
{
  extern int LeftXScreen;
  extern int FontMaxWidth;
  int offset,margin;

  margin = LeftXScreen - FontMaxWidth;
  if (margin < 0) margin = 0;
 
  offset = LeftXScreen - margin;
  LeftXScreen = margin;

  hwnd->Cursor->cursor_off();
  Scroll(hwnd->Text, offset, 1);
  SetCursor(hwnd);
}


/*
** Funzione per spostarsi a destra di un carattere.
** L'handler punta alla finestra client.
*/

void LineRight(MainWindow *hwnd)
{
  extern int LeftXScreen;
  extern int MaxXScreen;
  extern int FontMaxWidth;

  int offset,margin;

  margin = LeftXScreen + FontMaxWidth;
  if (margin > MaxXScreen) margin = MaxXScreen;

  offset = LeftXScreen - margin;
  LeftXScreen = margin;

  hwnd->Cursor->cursor_off();
  Scroll(hwnd->Text, offset, 1);
  SetCursor(hwnd);
}
