/***************************************************************************
 *                                                                         *
 *    File:         TeDSelect.cpp                                          *
 *                                                                         *
 *    Description:  Selection X-Window TeD                                 *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Date:         29 Aug 1994                                            *
 *                                                                         *
 ***************************************************************************/

#include <qrect.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "TeDFrame.h"
#include "TeD.h"

/*
** Funzione per aggiornare le informazioni sull'area selezionata. Forza un
** repainting dello schermo se necessario.
*/

void TextSelect(MainWindow *hwnd, char fSel, int x, int y, char fOutside)
{
  extern char fmark;
  extern unsigned int mark_row_end;
  extern unsigned int mark_line_end;
  extern unsigned int mark_column_end;
  extern struct row *mark_pointer_end;
  extern unsigned int cursor_row;
  extern unsigned int cursor_column;
  extern struct row *actual;
  extern QRect rcClient;
  extern int FontMaxWidth;
  extern int FontDescent;
  extern int FontHeight;

  if (fSel && fOutside) {  /* Attiva scorrimento schermo */

     if (x < rcClient.left() + FontMaxWidth) {
        x = rcClient.left() + FontMaxWidth;
        LineLeft(hwnd);
        }
     else if (x > rcClient.right() - FontMaxWidth) {
        x = rcClient.right() - FontMaxWidth;
        LineRight(hwnd);;
        }
     if (y < rcClient.top() + FontDescent) {
        LineUp(hwnd);
         }
     else if (y > rcClient.bottom() - FontDescent) {
        y = rcClient.bottom() + 1;
        LineDown(hwnd);
        }
     }

  if (fSel != fmark) {

     /* Invertire lo stato della selezione */

     fmark = fSel;
     if (fSel) {    /* Aggiorna le coordinate dell'area selezionata */

        GetCharAt(x, y, &mark_row_end, &mark_line_end, &mark_column_end,
                  &mark_pointer_end);
        }

     /* Invalida la regione fra il cursore e il marcatore di selezione */
     InvalidateArea(hwnd, cursor_row, cursor_column, actual, mark_row_end,
                    mark_column_end, mark_pointer_end);
     }

  else if (fSel) {

     /* Estendere la selezione */

     unsigned int old_row;
     unsigned int old_column;
     struct row *old_pointer;

     old_row = mark_row_end;
     old_column = mark_column_end;
     old_pointer = mark_pointer_end;

     /* Aggiorna le coordinate dell'area selezionata */
     GetCharAt(x, y, &mark_row_end, &mark_line_end, &mark_column_end,
               &mark_pointer_end);

     /* Invalida la regione fra il nuovo e il vecchio marcatore di selezione */
     if (mark_row_end != old_row || mark_column_end != old_column)
        InvalidateArea(hwnd, old_row, old_column, old_pointer, mark_row_end,
                       mark_column_end, mark_pointer_end);
     }

  /* Controllo esistenza reale area selezionata */

  if (fmark) {
     if (cursor_row == mark_row_end && cursor_column == mark_column_end)
        fmark = FALSE;
     else if (cursor_row == mark_row_end + 1 && cursor_column == 0 &&
              mark_column_end == strlen(mark_pointer_end->text) + 1)
        fmark = FALSE;
     }
}


/*
** Funzione per invalidare un'area compresa fra due righe e due colonne.
*/

void InvalidateArea(MainWindow *hwnd, unsigned int row_start,
                    unsigned int column_start,
                    struct row *point_start, unsigned int row_stop,
                    unsigned int column_stop, struct row *point_stop)
{
  extern unsigned int first_video_row;
  extern unsigned int last_video_row;
  extern struct row *video;
  unsigned int buffer;
  unsigned int frow;
  unsigned int lrow;
  char disp[ROWLEN+2];

  /* Ordina la regione da invalidare */
  if (row_start > row_stop) {
     buffer = row_start;
     row_start = row_stop;
     row_stop = buffer;
     buffer = column_start;
     column_start = column_stop;
     column_stop = buffer;
     point_start = point_stop;
     }
  else if (row_start == row_stop && column_start > column_stop) {
     buffer = column_start;
     column_start = column_stop;
     column_stop = buffer;
     }

  /* Limita l'aggiornamento alle righe visualizzate */

  frow = row_start;
  if (frow < first_video_row) {
     frow = first_video_row - 1;
     point_start = video->prev;
     }

  lrow = (row_stop > last_video_row) ? last_video_row + 1 : row_stop;
 
  /* Invalida le righe */
  for (buffer = frow; buffer <= lrow; ++buffer) {

     strcpy(disp,point_start->text);
     if (point_start->carriage == soft)
       strcat(disp,SOFTRETURN);
     else if (point_start->carriage == hard)
       strcat(disp,HARDRETURN);
     else
       strcat(disp,CONTINUATION);

     VideoRegionUpdate(buffer,disp,hwnd,(buffer > row_start) ? 0 :
                       column_start,
                       (buffer < row_stop) ? strlen(point_start->text) + 1:
                       column_stop);
     point_start = point_start->next;
     }
}

/*
** Funzione per annullare la selezione se presente. <hwnd> punta
** alla finestra client.
** Ritorna lo stato della selezione prima dell'operazione.
*/

char Deselect(MainWindow *hwnd)
{
  extern char fmark;
  extern unsigned int mark_row_end;
  extern unsigned int mark_column_end;
  extern struct row *mark_pointer_end;
  extern unsigned int cursor_row;
  extern unsigned int cursor_column;
  extern struct row *actual;

  if (fmark) {

    fmark = FALSE;
 
    /* Invalida la regione fra il cursore e il marcatore di selezione */
    InvalidateArea(hwnd, cursor_row, cursor_column, actual, mark_row_end,
                   mark_column_end, mark_pointer_end);
    return(TRUE);
    }
  else
    return(FALSE);
}


/*
** Funzione per attivare la selezione. Non vengono effettuati controlli
** sulla eventuale selezione precedente.
*/

void Select(MainWindow *hwnd)
{
  extern char fmark;
  extern unsigned int mark_row_end;
  extern unsigned int mark_column_end;
  extern struct row *mark_pointer_end;
  extern unsigned int cursor_row;
  extern unsigned int cursor_column;
  extern struct row *actual;

  fmark = TRUE;

  /* Invalida la regione fra il cursore e il marcatore di selezione */

  InvalidateArea(hwnd, cursor_row, cursor_column, actual, mark_row_end,
                 mark_column_end, mark_pointer_end);
}


/*
** Funzione per selezionare una parola in seguito ad un doppio click del
** pulsante sinistro del mouse. Sposta il cursore all'inizio della selezione.
** Aggiorna lo schermo.
*/

void WordSelect(MainWindow *hwnd, int x, int y)
{
  extern char fmark;
  extern unsigned int mark_row_end;
  extern unsigned int mark_line_end;
  extern unsigned int mark_column_end;
  extern struct row *mark_pointer_end;
  extern unsigned int cursor_row;
  extern unsigned int cursor_line;
  extern unsigned int cursor_column;
  extern struct row *actual;
  char temp;
  enum ret car;

  /* Deseleziona testo se presente */
  Deselect(hwnd);

  /* Posiziona il puntatore dell'area marcata sul cursore */
  mark_row_end = cursor_row;
  mark_line_end = cursor_line;
  mark_column_end = cursor_column;
  mark_pointer_end = actual;

  /* Ricerca primo carattere non blank dal carattere selezionato a sinistra */

  temp = *(actual->text + cursor_column);  /* carattere selezionato */
  if (temp == (char)NULL)                  /* fine riga */
    temp = (char)32;
  while (temp == (char)32) {
     temp = PrecCharPar(&cursor_row, &cursor_line, &cursor_column, &actual);
     }

  /* Ricerca primo carattere con alla sinistra un blank o 
     l'inizio del paragrafo */

  while (temp != (char)32 && temp!= (char)NULL) {
     temp = cursor_column > 0 ? *(actual->text + cursor_column - 1) :
       ((actual->prev ? actual->prev->carriage : hard) == cont) ? 
       (char)NULL : (char)32;
     if (temp != (char)32)
       temp = PrecCharPar(&cursor_row, &cursor_line, &cursor_column, &actual);
     }

  /* Ricerca primo carattere blank alla destra del carattere selezionato */

  temp = *(mark_pointer_end->text + mark_column_end);
  if (temp == (char)NULL) {                           /* fine riga */
     if (mark_pointer_end->carriage == soft)
        temp = (char)32;
     else
        temp = (char)33;          /* qualunque carattere != NULL e != 32 */
     }
  while (temp != (char)32 && temp != (char)NULL) {
     temp = NextCharPar(&mark_row_end, &mark_line_end, &mark_column_end,
                        &mark_pointer_end);
     }

  /* Ricerca primo carattere non blank alla destra del carattere */

  while (temp == (char)32) {
     temp = NextCharPar(&mark_row_end, &mark_line_end, &mark_column_end,
                        &mark_pointer_end);
     }

  // Switch on cursor
  SetCursor(hwnd);

  /* Controllo reale esistenza selezione */

  if (cursor_row == mark_row_end && cursor_column == mark_column_end)
    return;
  else if (cursor_row == mark_row_end + 1 && cursor_column == 0 &&
           mark_column_end == strlen(mark_pointer_end->text) + 1)
    return;

  /* Seleziona parola trovata */

  Select(hwnd);
}

/*
** Funzione per spostarsi al carattere precedente all'interno di uno
** stesso paragrafo.
** Ritorna il carattere su cui si e' spostato. Se si raggiunge la fine
** del paragrafo, non si sposta e ritorna (char)NULL. Per il buon
** funzionamento del programma e' necessario che la colonna non ecceda
** il valore della lunghezza della stringa puntata da *now.
*/

char PrecCharPar(unsigned int *row, unsigned int *line, unsigned int *column,
                 struct row **now)
{
  if (*column > 0) {                    /* non e' la prima colonna */
    --*column;
    return(*((*now)->text + *column));
    }
  else {    /* prima colonna */
    enum ret car;              
    if ((car=((*now)->prev ? (*now)->prev->carriage : hard)) != hard) {
      /* arretra */
      *now = (*now)->prev;
      --*row;
      *column = strlen((*now)->text);
      if (car == cont) {
         --*column;
         return(*((*now)->text + *column));
         }
      else {
        --*line;
        return((char)32);
        }
      }
    else   /* non arretra */
      return((char)NULL);
    }
}


/*
** Funzione per spostarsi al carattere successivo all'interno di uno
** stesso paragrafo.
** Ritorna il carattere su cui si e' spostato. Se si raggiunge la fine
** del paragrafo, non si sposta e ritorna (char)NULL. Per il buon
** funzionamento della funzione e' necessario che la colonna in ingresso
** non ecceda il valore della lunghezza della stringa puntata da *now.
*/


char NextCharPar(unsigned int *row, unsigned int *line, unsigned int *column,
                 struct row **now)
{
  unsigned int len;                     /* variabile ausiliaria */
  enum ret car;                         /* tipo ritorno carrello */
  len = strlen((*now)->text);           /* lunghezza riga attuale */
  car = (*now)->carriage;               /* tipo ritorno carrello */

  if (*column + 1 < len) {              /* non e' alla fine della riga */
     ++*column;
     return(*((*now)->text + *column));
     }
  else if (*column + 1 == len) {        /* deve andare alla fine della riga */
     if (car == hard) {            /* hard return: si sposta e ritorna NULL */
       ++*column;
       return((char)NULL);
       }
     else if (car == soft) {      /* soft return: si sposta e ritorna spazio */
       ++*column;
       return((char)32);
       }
     else if (car == cont) {     /* continuation mark: va alla riga seguente */
       *column = 0;
       ++*row;
       *now = (*now)->next;
       return(*((*now)->text));
       }
     }
  else {                                /* e' gia' alla fine della riga */
     if (car != hard) {                 /* va alla riga seguente */
       *column = 0;
       ++*row;
       if (car == soft)
         ++*line;
       *now = (*now)->next;
       return(*((*now)->text));
       }
     else {                 /* hard return: non si sposta e ritorna NULL */
       return((char)NULL); 
       }
     }
}