/***************************************************************************
 *                                                                         *
 *    File:         TeDSubstitute.cpp                                      *
 *                                                                         *
 *    Description:  Substitute manager of X-Window TeD                     *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  All rights reserved.                                   *
 *                                                                         *
 *    Date:         1 Jan 1996                                             *
 *                                                                         *
 ***************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <qlabel.h>                      // Labels
#include <qdialog.h>                     // Dialogs
#include <qframe.h>                      // Frames
#include <qpushbt.h>                     // Push buttons
#include <qchkbox.h>                     // Check boxes
#include <qradiobt.h>                    // Radio buttons
#include <qbttngrp.h>                    // Button groups
#include <qlined.h>                      // Line edit
#include "TeDFrame.h"
#include "TeD.h"


// Substitute window constructor

WinSubs::WinSubs(QWidget* parent, const char* name, WFlags f)
        : QDialog (0, name, TRUE, f)
{

  // Save parent
  hwnd = parent;

  // Sets frame & separator
  frame = new QFrame(this);
  separator = new QFrame(this);
  frame->setFrameStyle( QFrame::Panel | QFrame::Raised );
  separator->setFrameStyle( QFrame::HLine | QFrame::Sunken );

  // Sets label
  Mlabel1 = new QLabel(this);
  Mlabel1->setText("Replace: ");
  Mlabel1->setAlignment(AlignCenter);
  Mlabel1->setAutoResize(TRUE);
  Mlabel2 = new QLabel(this);
  Mlabel2->setText("With: ");
  Mlabel2->setAlignment(AlignCenter);
  Mlabel2->setAutoResize(TRUE);
  
  // Sets line edits
  line1 = new QLineEdit(this);
  line1->setMaxLength(ROWLEN);                    // Max ROWLEN characters
  line1->setText(((MainWindow *) hwnd)->rtext);   // Copy text
  line2 = new QLineEdit(this);
  line2->setMaxLength(ROWLEN);                    // Max ROWLEN characters
  line2->setText(((MainWindow *) hwnd)->stext);   // Copy text

  // Sets buttons
  Ok = new QPushButton(this);
  Cancel = new QPushButton(this);
  Yes = new QPushButton(this);
  All = new QPushButton(this);
  No = new QPushButton(this);
  case_sens = new QCheckBox(this);
  dirbut = new QButtonGroup("Direction", this);
  rb[0] = new QRadioButton(dirbut);
  rb[1] = new QRadioButton(dirbut);

  Ok->setText("Ok");
  Cancel->setText("Cancel");
  Yes->setText("Yes");
  All->setText("All");
  No->setText("No");
  case_sens->setText("Case sensitive");
  rb[0]->setText("Up");
  rb[1]->setText("Down");

  // Create timer for all substitution
   timer = new QTimer(this);
   connect( timer, SIGNAL(timeout()), SLOT(yes_clicked ()));

  // Connect button signals
  connect(Ok, SIGNAL(clicked()), SLOT(ok_clicked()));
  connect(Cancel, SIGNAL(clicked()), SLOT(cancel_clicked()));
  connect(Yes, SIGNAL(clicked()), SLOT(yes_clicked()));
  connect(All, SIGNAL(clicked()), SLOT(all_clicked()));
  connect(No, SIGNAL(clicked()), SLOT(no_clicked()));
  connect(case_sens, SIGNAL(clicked()), SLOT(case_pressed()));
  connect(dirbut, SIGNAL(clicked(int)), SLOT(direction(int)));

  // Connect find signal
  connect (this, SIGNAL(answer(find_text)), parent,
           SLOT(FindAnswer(find_text)));

  // Check down direction button
  rb[1]->setChecked(TRUE);

  // Check case sensitive button
  case_sens->setChecked(TRUE);

  // Dim Yes All No buttons
  Yes->setEnabled(FALSE);
  All->setEnabled(FALSE);
  No->setEnabled(FALSE);

  // Focus to substitute line editor
  line1->setFocus();

  // Initialize data
  ft.case_sensitive = TRUE;
  ft.down_search = TRUE;

  // Resize window & show
  resize(365, 265);
  show();
}

// Substitute window destructor

WinSubs::~WinSubs()
{
  // Free storage

  delete Mlabel1;                // Message box label
  delete Mlabel2;                // Message box label
  delete line1;                  // Line edit
  delete line2;                  // Line edit
  delete case_sens;              // Case sensitive check box
  delete[] rb;                   // Radio buttons
  delete dirbut;                 // Direction button group
  delete Ok;                     // Ok button
  delete Cancel;                 // Cancel button
  delete Yes;                    // Yes button
  delete All;                    // All button
  delete No;                     // No button
  delete separator;              // Separator line
  delete frame;                  // Frame
  delete timer;                  // Timer
}


// Substitute window resize event

void WinSubs::resizeEvent( QResizeEvent* )
{
  const int space = 20;
  const int sspace = 10;
  const int bspace = 30;
  const int bbspace = 50;
  const int hei = 25;
  int blen = (this->width() - 4 * bspace) / 3;

  // Sets frame geometry
  frame->setGeometry(0, 0, this->width(), this->height());

  // Sets separator geometry
  separator->setGeometry(0, this->height() - bspace - sspace - hei - hei,
                         this->width(), 4);  

  // Sets label 1 geometry
  Mlabel1->setGeometry(sspace, sspace, 60, hei);

  // Sets line 1 edit geometry
  line1->setGeometry(space + 50, sspace, this->width() - space - bspace - 50, 
                    hei);

  // Sets label 2 geometry
  Mlabel2->setGeometry(sspace, bspace + hei, 60, hei);

  // Sets line 2 edit geometry
  line2->setGeometry(space + 50, bspace + hei, this->width() - space
                     - bspace - 50, hei);

  // Sets case sensitive geometry
  case_sens->setGeometry(space, bspace + sspace + hei + hei, 120, hei);

  // Sets button group geometry
  dirbut->setGeometry(this->width() - 100 - space, bspace + sspace + hei + hei,
                      100, 3 * hei);
  rb[0]->setGeometry(sspace, space, 50, hei);
  rb[1]->setGeometry(sspace, space + hei, 50, hei);

  // Sets ok cancel buttons geometry
  Ok->setGeometry ((width() - bspace) / 2 - blen,
                   this->height() - space - hei - hei, blen, hei);

  Cancel->setGeometry ((width() + bspace) / 2,
                       this->height() - space - hei - hei,
                       blen, hei);

  // Sets yes all no buttons geometry
  Yes->setGeometry (bspace, this->height() - sspace - hei, blen, hei);
  No->setGeometry (bspace + bspace + blen,
                   this->height() - sspace - hei, blen, hei);
  All->setGeometry (this->width() - bspace - blen,
                    this->height() - sspace - hei, blen, hei);

}

// Ok button pressed

void WinSubs::ok_clicked()
{
  extern unsigned int cursor_row;
  extern unsigned int cursor_line;
  extern unsigned int cursor_column;
  extern struct row *actual;

  // Initialize data
  nsubs = 0;
  fAll = FALSE;

  // Initialize search structure
  ft.start_row = cursor_row;
  ft.start_line = cursor_line;
  ft.start_column = cursor_column;
  ft.start_pointer = actual;
  strcpy(ft.text,line1->text());

  // Copy find and replace text to main application buffer
  strcpy(((MainWindow *) hwnd)->rtext, line1->text());
  strcpy(((MainWindow *) hwnd)->stext, line2->text());

  // Disable line edits and buttons
  line1->setEnabled(FALSE);
  line2->setEnabled(FALSE);
  Ok->setEnabled(FALSE);
  case_sens->setEnabled(FALSE);
  rb[0]->setEnabled(FALSE);
  rb[1]->setEnabled(FALSE);  

  // Call search routine
  if (search_string(&ft)) {
    emit answer(ft);

    // Enable yes all no buttons
    Yes->setEnabled(TRUE);
    All->setEnabled(TRUE);
    No->setEnabled(TRUE);
    }
  else
    cancel_clicked();
}


// Cancel button pressed

void WinSubs::cancel_clicked()
{

  // Check if want to close or to stop search
  if (!Ok->isEnabled()) {

    // Timer stop
    timer->stop();

    // Enable line edits and buttons
    line1->setEnabled(TRUE);
    line2->setEnabled(TRUE);
    Ok->setEnabled(TRUE);
    case_sens->setEnabled(TRUE);
    rb[0]->setEnabled(TRUE);
    rb[1]->setEnabled(TRUE);

    // Show replace results
    ReplaceResult();

    // Disable yes all no buttons
    Yes->setEnabled(FALSE);
    All->setEnabled(FALSE);
    No->setEnabled(FALSE);
    }

  else
    // Close window
    close (FALSE);
}


// Yes button pressed

void WinSubs::yes_clicked()
{
  char temp[ROWLEN + 1];

  // Switch off cursor
  ((MainWindow *)hwnd)->Cursor->cursor_off();

  // Substitute marked text
  strcpy(temp, line2->text());
  DelMark((MainWindow *)hwnd, temp);

  // Advance number of substitution
  ++nsubs;

  // Find new string
  if (search_string(&ft)) {
    emit answer(ft);
    // Start single shot timer if all substitution is in progress
    if (fAll) {
      timer->start(20, TRUE);
      }
    }
  else
    cancel_clicked();
}


// No button pressed

void WinSubs::no_clicked()
{

  // Start new search
  if (search_string(&ft))
    emit answer(ft);
  else
    cancel_clicked();
}

// All button pressed

void WinSubs::all_clicked()
{
  // Disable yes all no buttons
  Yes->setEnabled(FALSE);
  No->setEnabled(FALSE);
  All->setEnabled(FALSE);

  // Signal all substitution
  fAll = TRUE;

  // Call yes button function
  yes_clicked();
}


// Case sensitive button pressed

void WinSubs::case_pressed ()
{

  ft.case_sensitive = case_sens->isChecked();

}

// Direction changed

void WinSubs::direction (int i)
{

  ft.down_search = i;

}


// Substitute results visualization

void WinSubs::ReplaceResult()
{
  char temp[256];
  sprintf(temp, "Replace result: Made %d substitutions", nsubs);

  ((MainWindow *)hwnd)->MessageBox(temp, MB_OK);

}
