/******************************************************************
************************* Text class ******************************
******************************************************************/

#include <qwidget.h>                       // Widget class
#include <qevent.h>                        // Event handler
#include <qfont.h>                         // Fonts
#include "TeDFrame.h"
#include "TeD.h"


// Text widget constructor

TextWidget::TextWidget(QWidget* parent, const char* name, WFlags f)
              : QWidget (parent, name, f)
{
    // Sets background color
   setBackgroundColor(white);

    // Create text cursor
    ((MainWindow *)parent)->Cursor = new TextCursor(parent);

   // Creates timer for scrolling selection
   timer = new QTimer(this);
   connect( timer, SIGNAL(timeout()), SLOT(ScrollSelection ()));

   // Update window
   repaint();
}


// Painting event of the text widget
void TextWidget::paintEvent(QPaintEvent *e)
{
    QPainter p;
    QRect rect;
    extern struct row *video;             // pointer to first video row
    extern unsigned int first_video_row;  // number first video row
    extern unsigned int last_video_row;   // number first video row
    extern unsigned int t_len;            // lenght of text  
    extern unsigned int t_line;           // last line of text
    extern char fmark;                    // marked text flag
    extern unsigned int cursor_row;       // cursor row
    extern unsigned int cursor_column;    // cursor column
    extern unsigned int mark_row_end;     // row end marked text
    extern unsigned int mark_column_end;  // column end marked text 
    extern int LeftXScreen;               // left X screen coordinate
    extern int MaxXScreen;                // right X screen coordinate
    extern QRect rcClient;                // window text
    extern int FontHeight;                // Font height
    extern int FontAscent;                // Font ascent
    extern int FontDescent;               // Font descent
    extern int FontMaxWidth;              // Font max width
    extern int WidthTable[];              // Font width table
    extern char fVertTrack;               // vertical tracking
    extern char fHorzTrack;               // horizontal tracking
    extern char fInitialGoto;             // initial goto
    extern char fontname[];               // font to be used
    extern int size;                      // size of the font
    unsigned int begin_row;               // row begin hilight
    unsigned int begin_column;            // column end hilight
    unsigned int end_row;                 // row end hilight
    unsigned int end_column;              // column end hilight
    register int i;                       // counter

    struct row *pointer;                  // auxiliary pointer
    char disp[ROWLEN+2];                  // row to display
    QPoint pt;                            // starting point to draw string

    // Begin painting
    p.begin(this);

    // Sets font and size to be used
    QFont font(fontname,size);
    p.setFont(font);

    // Initialize pointer
    pointer = video;

    // Starting point and horizontal limits of window
    pt.setX(rcClient.left() + FontMaxWidth - LeftXScreen);
    pt.setY(rcClient.top());
    rect.setLeft(rcClient.left());
    rect.setRight(rcClient.right());

    MaxXScreen = 0;
    last_video_row = first_video_row;

    if (fmark) {

     // order region to hilight
     if (cursor_row < mark_row_end) {
        begin_row = cursor_row;
        end_row = mark_row_end;
        begin_column = cursor_column;
        end_column = mark_column_end;
        }
     else if (cursor_row > mark_row_end) {
        begin_row = mark_row_end;
        end_row = cursor_row;
        begin_column = mark_column_end;
        end_column = cursor_column;
        }
     else if (cursor_column < mark_column_end) {
        begin_row = end_row = cursor_row;
        begin_column = cursor_column;
        end_column = mark_column_end;
        }
     else {
        begin_row = end_row = cursor_row;
        begin_column = mark_column_end;
        end_column = cursor_column;
        }
     }

    if (pt.y() - FontAscent <= e->rect().bottom() &&
        pt.y() + FontDescent >= e->rect().top())
      {
      if (pointer->prev)
        DisplayLine(&p,&pt,&(e->rect()),pointer->prev,disp,fmark &&
               (first_video_row - 1 >= begin_row) &&
               (first_video_row - 1 <= end_row),
               (first_video_row - 1 == begin_row) ?
               begin_column : 0,
               (first_video_row - 1 == end_row) ? end_column :
               strlen(pointer->prev->text) + 1);

      else {
         rect.setTop(rcClient.top());
         rect.setBottom(rcClient.top() + FontHeight - FontAscent);
         p.eraseRect(rect);
         }
      }
  pt.setY(pt.y() + FontHeight);

  do {
    if (pt.y() - FontAscent <= e->rect().bottom() &&
        pt.y() + FontDescent >= e->rect().top())
      DisplayLine(&p,&pt,&(e->rect()),pointer,disp,fmark
                && (last_video_row >= begin_row)
                && (last_video_row <= end_row), (last_video_row == begin_row) ?
                begin_column : 0, (last_video_row == end_row) ? end_column :
                strlen(pointer->text) + 1);
    else {
      // Copy string with terminator
      strcpy(disp,pointer->text);
      if (pointer->carriage == soft)
        strcat(disp,SOFTRETURN);
      else if (pointer->carriage == hard)
        strcat(disp,HARDRETURN);
      else
        strcat(disp,CONTINUATION);
      }
    
    ComputeLineWidth(disp,&MaxXScreen);
    pointer = pointer->next;
    ++last_video_row;
    pt.setY(pt.y() + FontHeight);
  } while ( pt.y() <= rcClient.bottom() - FontDescent && pointer);

  if (pt.y() - FontAscent <= e->rect().bottom() &&
      pt.y() + FontDescent >= e->rect().top())
    {
    if (pointer)
      DisplayLine(&p,&pt,&(e->rect()),pointer,disp,fmark
                && (last_video_row >= begin_row)
                && (last_video_row <= end_row), (last_video_row == begin_row) ?
                begin_column : 0, (last_video_row == end_row) ? end_column :
                strlen(pointer->text) + 1);

    else {
       rect.setTop(pt.y() - FontAscent);
       rect.setBottom(rcClient.bottom());
       p.eraseRect(rect);
       }
    }
  --last_video_row;
  MaxXScreen += WidthTable['m' & 0xFF];

  // End painting
  p.end();

  // Adjust limits of scrollbars
  if (!fVertTrack)
    {
     ((MainWindow *)parentWidget())->vscroll->setRange(0,(int)t_len);
     ((MainWindow *)parentWidget())->vscroll->setSteps (1,
                     (int)last_video_row - (int)first_video_row + 1);
     ((MainWindow *)parentWidget())->vscroll->setValue(first_video_row);
    }

  if (!fHorzTrack)
    {
     ((MainWindow *)parentWidget())->hscroll->setRange(0, MaxXScreen);
     ((MainWindow *)parentWidget())->hscroll->setSteps
                                  (FontMaxWidth, MaxXScreen / 3);
     ((MainWindow *)parentWidget())->hscroll->setValue(LeftXScreen);
    }

  if (fInitialGoto) {                  // Move window
    fInitialGoto = FALSE;
    CentrateCursor(((MainWindow *)parentWidget()));
    }
  else

    // Update cursor visualization
    ((MainWindow *)parentWidget())->Cursor->cursor_fill();
}
