/***************************************************************************
 *                                                                         *
 *    File:         TeDUpdate.cpp                                          *
 *                                                                         *
 *    Description:  Video update of X-Window TeD                           *
 *                                                                         *
 *    Copyright:    (C) Copyright Paolo Galatola 1994.                     *
 *                  Tutti i diritti riservati.                             *
 *                                                                         *
 *    Date:         29 Aug 1994                                            *
 *                                                                         *
 ***************************************************************************/


#include <stdlib.h>
#include <stdio.h>
#include <qrect.h>
#include "TeDFrame.h"
#include "TeD.h"


/*
** Funzione per invalidare il rettangolo occupato da una riga a video.
** <nline> contiene il numero della linea da invalidare.
** <hwnd> punta alla finestra client.
** <first_modified> indica la prima colonna modificata.
*/

void VideoUpdate(unsigned int nline, char *string,
                 MainWindow *hwnd, unsigned int first_modified)
{
  extern unsigned int first_video_row;     // First video row
  extern QRect rcClient;                   // Client window rectangle
  extern int LeftXScreen;                  // Left margin screen
  extern int FontHeight;                   // Font height
  extern int FontAscent;                   // Font ascent
  extern int FontDescent;                  // Font descent
  extern int WidthTable[];                 // Font width table
  QRect rcUpdate;                          // rectangle to invalidate
  unsigned int i;                          // auxiliary counter

  rcUpdate.setLeft(rcClient.top() + ((int)nline - (int)first_video_row + 1L)*
                   FontHeight);
  rcUpdate.setTop(rcUpdate.left() - FontAscent);
  rcUpdate.setBottom(rcUpdate.left() + FontDescent);

  if (rcUpdate.top() < rcClient.top()) rcUpdate.setTop(rcClient.top());
  if (rcUpdate.bottom() > rcClient.bottom()) 
    rcUpdate.setBottom(rcClient.bottom());
  if (rcUpdate.top() < rcUpdate.bottom()) {

      // Note: the left corner is FontMaxWidth more at left than
      // the left border to allow for characters going into
      // the left margin as the backslash
      rcUpdate.setLeft(rcClient.left() - LeftXScreen);
      for (i = 0; i < first_modified; ++i)
         rcUpdate.setLeft(rcUpdate.left() + WidthTable[*(string + i) & 0xFF]);
      rcUpdate.setRight(rcClient.right());
      hwnd->Text->repaint(rcUpdate);
    }
}

/*
** Funzione per invalidare il rettangolo occupato da una riga a video.
** <nline> contiene il numero della linea da invalidare.
** <hwnd> punta alla finestra client.
** <first_modified> indica la prima colonna modificata, <last_modified>
** indica l'ultima colonna modificata.
**
** N.B. Per il corretto funzionamento deve essere <last_modified> >=
** <first_modified>.
*/

void VideoRegionUpdate(unsigned int nline, char *string,
                       MainWindow *hwnd, unsigned int first_modified,
                       unsigned int last_modified)
{
  extern unsigned int first_video_row;    // First video row
  extern QRect rcClient;                  // Client window rectangle
  extern int LeftXScreen;                 // Left margin screen
  extern int FontHeight;                  // Font height
  extern int FontAscent;                  // Font ascent
  extern int FontDescent;                 // Font descent
  extern int FontMaxWidth;                // Font max width
  extern int WidthTable[];                // Font width table
  QRect rcUpdate;                         // Rectangle to invalidate
  unsigned int i;                         // Auxiliary counter

  rcUpdate.setLeft(rcClient.top() + ((int)nline - (int)first_video_row + 1L)*
                   FontHeight);
  rcUpdate.setTop(rcUpdate.left() - FontAscent);
  rcUpdate.setBottom(rcUpdate.left() + FontDescent);

  if (rcUpdate.top() < rcClient.top())
    rcUpdate.setTop(rcClient.top());
  if (rcUpdate.bottom() > rcClient.bottom())
    rcUpdate.setBottom(rcClient.bottom());
  if (rcUpdate.top() < rcUpdate.bottom() && last_modified > first_modified) {
  
    // The left and right border are suitably increased for
      // characters going out of their natural rectangle
      rcUpdate.setLeft(rcClient.left() - LeftXScreen);
      for (i = 0; i < first_modified; ++i)
         rcUpdate.setLeft(rcUpdate.left() + WidthTable[*(string + i) & 0xFF]);
      rcUpdate.setRight(rcUpdate.left() + FontMaxWidth + FontMaxWidth);
      for (i = first_modified; i <= last_modified; ++i)
         rcUpdate.setRight(rcUpdate.right() + WidthTable[*(string + i)&0xFF]);
      hwnd->Text->repaint(rcUpdate);
    }
}

/*
** Funzione per invalidare il rettangolo occupato sul video dalla riga
** <nline> fino al fondo dello schermo. <hwnd> punta alla finestra client.
** Se <nline> e' la prima riga, invalida tutto lo schermo.
*/

void DownUpdate(unsigned int nline, MainWindow *hwnd)
{
  extern unsigned int first_video_row;    // First video row
  extern QRect rcClient;                  // Client window rectangle
  extern int FontHeight;                  // Font height
  extern int FontAscent;                  // Font ascent
  QRect rcUpdate;                         // Rectangle to invalidate
  unsigned int i;                         // Auxiliary counter

  rcUpdate.setLeft(rcClient.left());
  rcUpdate.setRight(rcClient.right());
  rcUpdate.setTop(rcClient.top());

  if (nline != first_video_row)
    rcUpdate.setTop(rcUpdate.top() + ((int)nline - (int)first_video_row + 1L) *
                    FontHeight - FontAscent);

  rcUpdate.setBottom(rcClient.bottom());

  if (rcUpdate.top() <= rcUpdate.bottom()) {
      /* invalida finestra */
      if (rcUpdate.top() < rcClient.top())
         rcUpdate.setTop(rcClient.top());
      hwnd->Text->repaint(rcUpdate);
    }
}

/*
** Funzione per invalidare il rettangolo occupato sul video dalla riga
** <nstart> fino alla riga <nstop>. <hwnd> punta alla finestra client.
** Se <nstop> < <nstart> non invalida niente.
*/

void LinesUpdate(unsigned int nstart, unsigned int nstop,
                 MainWindow *hwnd)
{
  extern unsigned int first_video_row;     // First video row
  extern QRect rcClient;                   // Client window rectangle
  extern int FontHeight;                   // Font height
  extern int FontAscent;                   // Font ascent
  extern int FontDescent;                  // Font descent
  QRect rcUpdate;                          // Rectangle to invalidate
  unsigned int i;                          // Auxiliary counter

  if (nstop < nstart)                         /* esce */
    return;

  rcUpdate.setTop(rcClient.top() + ((int)nstart - (int)first_video_row + 1L) *
                  FontHeight - FontAscent);
 
  rcUpdate.setBottom(rcClient.top() + ((int)nstop - 
                    (int)first_video_row + 1L) * FontHeight + FontDescent);

  /* Controllo se la regione da invalidare cade nella finestra video */
  if (rcUpdate.top() > rcClient.bottom() || rcUpdate.bottom() < rcClient.top())
    return;

  /* Minimizzazione regione da invalidare */
  if (rcUpdate.top() < rcClient.top())
    rcUpdate.setTop(rcClient.top());
  if (rcUpdate.bottom() > rcClient.bottom())
    rcUpdate.setBottom(rcClient.bottom());

  /* invalida finestra */
  rcUpdate.setLeft(rcClient.left());
  rcUpdate.setRight(rcClient.right());
  hwnd->Text->repaint(rcUpdate);
}
