/***************************************************************************
******************************** TeDVideo.cpp ******************************
****************************************************************************/

// TeD video update

#include <qcolor.h>                  // Colors
#include <qpen.h>                    // Pens
#include <string.h>                  // String functions
#include <qpainter.h>                // Painting
#include <qpoint.h>                  // Points
#include <qrect.h> 	             // Rectangles
#include <qfont.h>                   // Fontmetrics
#include "TeDFrame.h"
#include "TeD.h"

/*
** Function to display a line with its attributes. p is the QPainter
** object used to draw the text. pt points to the starting point. ur points
** to the updating rectrangle. pointer is
** a pointer to the line to draw. In the string pointed by disp (of length
** ROWLEN + 2) it writes the actual string displayed, including the end
** of line character. If high is TRUE it hilights the columns between
** high_start and high_stop.
*/

void DisplayLine(QPainter *p, QPoint *pt, const QRect *ur, struct row *pointer,
                 char *disp, char high, unsigned int high_start,
                 unsigned int high_stop)
{
  int length;                               // String length
  extern QRect rcClient;                    // Client window rectangle
  extern int FontAscent;                    // Font ascent
  extern int FontDescent;                   // Font descent
  extern int FontHeight;                    // Font height
  QRect rcClear;                            // Clear rectangle
  QPoint pCurrent;                          // Current point
  int right;                                // Right point for filling

  // Clear beginning of line to draw

  rcClear.setTop(pt->y() - FontAscent);
  rcClear.setBottom(pt->y() + FontDescent);
  rcClear.setLeft(rcClient.left());
  rcClear.setRight(pt->x() - 1);
  p->eraseRect(rcClear);

  // Copy string to display
  strcpy(disp, pointer->text);
 
  // Add line terminator
  if (pointer->carriage == soft)
    strcat(disp,SOFTRETURN);
  else if (pointer->carriage == hard)
    strcat(disp,HARDRETURN);
  else
    strcat(disp,CONTINUATION);

  // Compute string length
  length = strlen(disp);

  // Copy starting point
  pCurrent = *pt;

  if (high) {           // Hilight active

     // Write beginning of string non hilighted
     if (high_start < length)
       PrintText(p, &pCurrent, ur, disp, (int) high_start);

     else {

       // No hilight at all
       PrintText(p, &pCurrent, ur, disp, length - 1);

       // Change color and draw end of line character
       p->setPen(darkGray);
       PrintText(p, &pCurrent, ur, disp + length - 1, 1);
       p->setPen(black);

       // Erase to the end and return
       rcClear.setLeft(pCurrent.x());
       rcClear.setRight(rcClient.right());
       p->eraseRect(rcClear);
       return;
       }

     // Determine bounding box of hilighted string
     QFontMetrics fm = p->fontMetrics();
     rcClear = fm.boundingRect (disp + high_start,
                                (int) high_stop - (int) high_start);
     rcClear.setRect(pCurrent.x(), pCurrent.y() - FontAscent, rcClear.width(),
                     FontHeight);

     if (rcClear.left() < ur->left())
        rcClear.setLeft(ur->left());

     right = rcClear.right();
     if (right > ur->right())
        rcClear.setRight(ur->right());

     // Fill bounding box with black
     if (rcClear.right() > rcClear.left())
       p->fillRect(rcClear, black);

     // Change color to white
     p->setPen(white);

     // Write hilighted string
     PrintText(p, &pCurrent, ur, disp + high_start,
                (int) high_stop - (int) high_start);

     // Change back color to black
     p->setPen(black);

     // Write end of non-hilighted string
     if (length > high_stop) {
       PrintText(p, &pCurrent, ur, disp + high_stop,
                  (int) length - (int) high_stop - 1L);

       // Change color and draw end of line character
       p->setPen(darkGray);
       PrintText(p, &pCurrent, ur, disp+length-1,1);
       p->setPen(black);
       }

     else {
       // Hilight to the end
       rcClear.setLeft(right + 1);
       rcClear.setRight(rcClient.right());
       p->fillRect(rcClear, black);
       return;
       }
     }

  else {        // No hilight

     PrintText(p, &pCurrent, ur, disp,(int)(length-1));

     // Change color and draw end of line character
     p->setPen(darkGray);
     PrintText(p, &pCurrent, ur, disp+length-1,1);
     p->setPen(black);
     }

  // Clear end of line
  rcClear.setLeft(pCurrent.x());
  rcClear.setRight(rcClient.right());
  p->eraseRect(rcClear);
}

/*
** Function to print text contained in the string text. Exactly num
** characters are printed. The starting point is incremented by the
** width of the text. The updating rectangle is checked against
** horizontal width only to determine which characters need redrawing.
*/
 
void PrintText(QPainter *p, QPoint *pt, const QRect *r, char *text, int num)
{
  int width;
  int i;
  extern int WidthTable[];

  for (i = 0;
       i < num;
       ++ i)
     {
     width = WidthTable[*(text+i) & 0xFF];
     if (pt->x() <= r->right() && pt->x() + width >= r->x())
       p->drawText(*pt, text + i, 1);
       pt->setX(pt->x() + width);
     }
}


/*
** Funzione per calcolare la larghezza a video di una stringa.
** In *width deve essere presente la larghezza "iniziale". Se la
** larghezza effettiva della linea e' maggiore di *width,
** *width viene aggiornata, se no non viene modificata. fm punta
** alla struttura metrica del font.
*/

void ComputeLineWidth(char *string,int *width)
{
  int i,parlen;
  extern int WidthTable[];

  parlen = 0;
  for (i = 0; i < strlen(string); ++i)
     parlen += WidthTable[*(string+i) & 0xFF];

  if (*width < parlen) *width =  parlen;
}
